// Code generated by go-swagger; DO NOT EDIT.

package storage

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// RootFSStorage Information about the storage used for the container's root filesystem.
//
// swagger:model RootFSStorage
type RootFSStorage struct {

	// Information about the snapshot used for the container's root filesystem.
	//
	Snapshot *RootFSStorageSnapshot `json:"Snapshot,omitempty"`
}
