// Code generated by sdkgen-v2. DO NOT EDIT.
package endpoints

import (
	"google.golang.org/protobuf/reflect/protoreflect"
)

var Aliases map[string]protoreflect.FullName = map[string]protoreflect.FullName{
	"ai":                   protoreflect.FullName("yandex.cloud.ai"),
	"container-registry":   protoreflect.FullName("yandex.cloud.containerregistry"),
	"dataproc":             protoreflect.FullName("yandex.cloud.dataproc"),
	"iam":                  protoreflect.FullName("yandex.cloud.iam"),
	"io-t":                 protoreflect.FullName("yandex.cloud.iot"),
	"k8s":                  protoreflect.FullName("yandex.cloud.k8s"),
	"kms":                  protoreflect.FullName("yandex.cloud.kms"),
	"load-balancer":        protoreflect.FullName("yandex.cloud.loadbalancer"),
	"mdb":                  protoreflect.FullName("yandex.cloud.mdb"),
	"marketplace":          protoreflect.FullName("yandex.cloud.marketplace"),
	"organization-manager": protoreflect.FullName("yandex.cloud.organizationmanager"),
	"quota-manager":        protoreflect.FullName("yandex.cloud.quotamanager"),
	"resource-manager":     protoreflect.FullName("yandex.cloud.resourcemanager"),
	"vpc":                  protoreflect.FullName("yandex.cloud.vpc"),
	"ydb":                  protoreflect.FullName("yandex.cloud.ydb"),
	"airflow":              protoreflect.FullName("yandex.cloud.airflow"),
	"apploadbalancer":      protoreflect.FullName("yandex.cloud.apploadbalancer"),
	"audittrails":          protoreflect.FullName("yandex.cloud.audittrails"),
	"backup":               protoreflect.FullName("yandex.cloud.backup"),
	"baremetal":            protoreflect.FullName("yandex.cloud.baremetal"),
	"billing":              protoreflect.FullName("yandex.cloud.billing"),
	"cdn":                  protoreflect.FullName("yandex.cloud.cdn"),
	"certificatemanager":   protoreflect.FullName("yandex.cloud.certificatemanager"),
	"cic":                  protoreflect.FullName("yandex.cloud.cic"),
	"cloudapps":            protoreflect.FullName("yandex.cloud.cloudapps"),
	"clouddesktop":         protoreflect.FullName("yandex.cloud.clouddesktop"),
	"cloudregistry":        protoreflect.FullName("yandex.cloud.cloudregistry"),
	"cloudrouter":          protoreflect.FullName("yandex.cloud.cloudrouter"),
	"compute":              protoreflect.FullName("yandex.cloud.compute"),
	"connectionmanager":    protoreflect.FullName("yandex.cloud.connectionmanager"),
	"datacatalog":          protoreflect.FullName("yandex.cloud.datacatalog"),
	"datasphere":           protoreflect.FullName("yandex.cloud.datasphere"),
	"datatransfer":         protoreflect.FullName("yandex.cloud.datatransfer"),
	"devtools":             protoreflect.FullName("yandex.cloud.devtools"),
	"dns":                  protoreflect.FullName("yandex.cloud.dns"),
	"endpoint":             protoreflect.FullName("yandex.cloud.endpoint"),
	"gitlab":               protoreflect.FullName("yandex.cloud.gitlab"),
	"kspm":                 protoreflect.FullName("yandex.cloud.kspm"),
	"loadtesting":          protoreflect.FullName("yandex.cloud.loadtesting"),
	"locator":              protoreflect.FullName("yandex.cloud.locator"),
	"lockbox":              protoreflect.FullName("yandex.cloud.lockbox"),
	"logging":              protoreflect.FullName("yandex.cloud.logging"),
	"logs":                 protoreflect.FullName("yandex.cloud.logs"),
	"metastore":            protoreflect.FullName("yandex.cloud.metastore"),
	"monitoring":           protoreflect.FullName("yandex.cloud.monitoring"),
	"operation":            protoreflect.FullName("yandex.cloud.operation"),
	"searchapi":            protoreflect.FullName("yandex.cloud.searchapi"),
	"securitydeck":         protoreflect.FullName("yandex.cloud.securitydeck"),
	"serverless":           protoreflect.FullName("yandex.cloud.serverless"),
	"servicename":          protoreflect.FullName("yandex.cloud.servicename"),
	"smartcaptcha":         protoreflect.FullName("yandex.cloud.smartcaptcha"),
	"smartwebsecurity":     protoreflect.FullName("yandex.cloud.smartwebsecurity"),
	"spark":                protoreflect.FullName("yandex.cloud.spark"),
	"speechsense":          protoreflect.FullName("yandex.cloud.speechsense"),
	"storage":              protoreflect.FullName("yandex.cloud.storage"),
	"trino":                protoreflect.FullName("yandex.cloud.trino"),
	"video":                protoreflect.FullName("yandex.cloud.video"),
	"ytsaurus":             protoreflect.FullName("yandex.cloud.ytsaurus"),
}
