package linodego

// Deprecated: AccountAvailabilityPagedResponse exists for historical compatibility and should not be used.
type AccountAvailabilityPagedResponse legacyPagedResponse[AccountAvailability]

// Deprecated: AccountBetasPagedResponse exists for historical compatibility and should not be used.
type AccountBetasPagedResponse legacyPagedResponse[AccountBetaProgram]

// Deprecated: BetaProgramPagedResponse exists for historical compatibility and should not be used.
type BetaProgramPagedResponse legacyPagedResponse[BetaProgram]

// Deprecated: DatabaseEnginesPagedResponse exists for historical compatibility and should not be used.
type DatabaseEnginesPagedResponse legacyPagedResponse[DatabaseEngine]

// Deprecated: DatabaseTypesPagedResponse exists for historical compatibility and should not be used.
type DatabaseTypesPagedResponse legacyPagedResponse[DatabaseType]

// Deprecated: DatabasesPagedResponse exists for historical compatibility and should not be used.
type DatabasesPagedResponse legacyPagedResponse[Database]

// Deprecated: DomainRecordsPagedResponse exists for historical compatibility and should not be used.
type DomainRecordsPagedResponse legacyPagedResponse[DomainRecord]

// Deprecated: DomainsPagedResponse exists for historical compatibility and should not be used.
type DomainsPagedResponse legacyPagedResponse[Domain]

// Deprecated: EventsPagedResponse exists for historical compatibility and should not be used.
type EventsPagedResponse legacyPagedResponse[Event]

// Deprecated: FirewallDevicesPagedResponse exists for historical compatibility and should not be used.
type FirewallDevicesPagedResponse legacyPagedResponse[FirewallDevice]

// Deprecated: FirewallsPagedResponse exists for historical compatibility and should not be used.
type FirewallsPagedResponse legacyPagedResponse[Firewall]

// Deprecated: ImagesPagedResponse exists for historical compatibility and should not be used.
type ImagesPagedResponse legacyPagedResponse[Image]

// Deprecated: InstanceConfigsPagedResponse exists for historical compatibility and should not be used.
type InstanceConfigsPagedResponse legacyPagedResponse[InstanceConfig]

// Deprecated: InstanceDisksPagedResponse exists for historical compatibility and should not be used.
type InstanceDisksPagedResponse legacyPagedResponse[InstanceDisk]

// Deprecated: InstanceFirewallsPagedResponse exists for historical compatibility and should not be used.
type InstanceFirewallsPagedResponse legacyPagedResponse[Firewall]

// Deprecated: InstanceVolumesPagedResponse exists for historical compatibility and should not be used.
type InstanceVolumesPagedResponse legacyPagedResponse[Volume]

// Deprecated: InstancesPagedResponse exists for historical compatibility and should not be used.
type InstancesPagedResponse legacyPagedResponse[Instance]

// Deprecated: InvoiceItemsPagedResponse exists for historical compatibility and should not be used.
type InvoiceItemsPagedResponse legacyPagedResponse[InvoiceItem]

// Deprecated: InvoicesPagedResponse exists for historical compatibility and should not be used.
type InvoicesPagedResponse legacyPagedResponse[Invoice]

// Deprecated: LKEClusterPoolsPagedResponse exists for historical compatibility and should not be used.
// LKEClusterPoolsPagedResponse represents a paginated LKEClusterPool API response.
type LKEClusterPoolsPagedResponse LKENodePoolsPagedResponse

// Deprecated: LKEVersionsPagedResponse exists for historical compatibility and should not be used.
type LKEVersionsPagedResponse legacyPagedResponse[LKEVersion]

// Deprecated: LKEClusterAPIEndpointsPagedResponse exists for historical compatibility and should not be used.
type LKEClusterAPIEndpointsPagedResponse legacyPagedResponse[LKEClusterAPIEndpoint]

// Deprecated: LKEClustersPagedResponse exists for historical compatibility and should not be used.
type LKEClustersPagedResponse legacyPagedResponse[LKECluster]

// Deprecated: LKENodePoolsPagedResponse exists for historical compatibility and should not be used.
type LKENodePoolsPagedResponse legacyPagedResponse[LKENodePool]

// Deprecated: IPAddressesPagedResponse exists for historical compatibility and should not be used.
type IPAddressesPagedResponse legacyPagedResponse[InstanceIP]

// Deprecated: IPv6PoolsPagedResponse exists for historical compatibility and should not be used.
type IPv6PoolsPagedResponse legacyPagedResponse[IPv6Range]

// Deprecated: IPv6RangesPagedResponse exists for historical compatibility and should not be used.
type IPv6RangesPagedResponse legacyPagedResponse[IPv6Range]

// Deprecated: LinodeKernelsPagedResponse exists for historical compatibility and should not be used.
type LinodeKernelsPagedResponse legacyPagedResponse[LinodeKernel]

// Deprecated: LinodeTypesPagedResponse exists for historical compatibility and should not be used.
type LinodeTypesPagedResponse legacyPagedResponse[LinodeType]

// Deprecated: LoginsPagedResponse exists for historical compatibility and should not be used.
type LoginsPagedResponse legacyPagedResponse[Login]

// Deprecated: LongviewClientsPagedResponse exists for historical compatibility and should not be used.
type LongviewClientsPagedResponse legacyPagedResponse[LongviewClient]

// Deprecated: LongviewSubscriptionsPagedResponse exists for historical compatibility and should not be used.
type LongviewSubscriptionsPagedResponse legacyPagedResponse[LongviewSubscription]

// Deprecated: MySQLDatabasesPagedResponse exists for historical compatibility and should not be used.
type MySQLDatabasesPagedResponse legacyPagedResponse[MySQLDatabase]

// Deprecated: NodeBalancersPagedResponse exists for historical compatibility and should not be used.
type NodeBalancersPagedResponse legacyPagedResponse[NodeBalancer]

// Deprecated: NodeBalancerConfigsPagedResponse exists for historical compatibility and should not be used.
type NodeBalancerConfigsPagedResponse legacyPagedResponse[NodeBalancerConfig]

// Deprecated: NodeBalancerNodesPagedResponse exists for historical compatibility and should not be used.
type NodeBalancerNodesPagedResponse legacyPagedResponse[NodeBalancerNode]

// Deprecated: NodeBalancerFirewallsPagedResponse exists for historical compatibility and should not be used.
type NodeBalancerFirewallsPagedResponse legacyPagedResponse[Firewall]

// Deprecated: NotificationsPagedResponse exists for historical compatibility and should not be used.
type NotificationsPagedResponse legacyPagedResponse[Notification]

// Deprecated: OAuthClientsPagedResponse exists for historical compatibility and should not be used.
type OAuthClientsPagedResponse legacyPagedResponse[OAuthClient]

// Deprecated: ObjectStorageKeysPagedResponse exists for historical compatibility and should not be used.
type ObjectStorageKeysPagedResponse legacyPagedResponse[ObjectStorageKey]

// Deprecated: ObjectStorageBucketsPagedResponse exists for historical compatibility and should not be used.
type ObjectStorageBucketsPagedResponse legacyPagedResponse[ObjectStorageBucket]

// Deprecated: ObjectStorageClustersPagedResponse exists for historical compatibility and should not be used.
type ObjectStorageClustersPagedResponse legacyPagedResponse[ObjectStorageCluster]

// Deprecated: PaymentsPagedResponse exists for historical compatibility and should not be used.
type PaymentsPagedResponse legacyPagedResponse[Payment]

// Deprecated: RegionsPagedResponse exists for historical compatibility and should not be used.
type RegionsPagedResponse legacyPagedResponse[Region]

// Deprecated: SSHKeysPagedResponse exists for historical compatibility and should not be used.
type SSHKeysPagedResponse legacyPagedResponse[SSHKey]

// Deprecated: TokensPagedResponse exists for historical compatibility and should not be used.
type (
	TokensPagedResponse legacyPagedResponse[Token]
	// Deprecated: RegionsAvailabilityPagedResponse exists for historical compatibility and should not be used.
	RegionsAvailabilityPagedResponse legacyPagedResponse[RegionAvailability]
)

// Deprecated: StackscriptsPagedResponse exists for historical compatibility and should not be used.
type StackscriptsPagedResponse legacyPagedResponse[Stackscript]

// Deprecated: TagsPagedResponse exists for historical compatibility and should not be used.
type TagsPagedResponse legacyPagedResponse[Tag]

// Deprecated: TaggedObjectsPagedResponse exists for historical compatibility and should not be used.
type TaggedObjectsPagedResponse legacyPagedResponse[TaggedObject]

// Deprecated: TicketsPagedResponse exists for historical compatibility and should not be used.
type TicketsPagedResponse legacyPagedResponse[Ticket]

// Deprecated: PostgresDatabasesPagedResponse exists for historical compatibility and should not be used.
type PostgresDatabasesPagedResponse legacyPagedResponse[PostgresDatabase]

// Deprecated: ProfileLoginsPagedResponse exists for historical compatibility and should not be used.
type ProfileLoginsPagedResponse legacyPagedResponse[ProfileLogin]

// Deprecated: UsersPagedResponse exists for historical compatibility and should not be used.
type UsersPagedResponse legacyPagedResponse[User]

// Deprecated: VolumesPagedResponse exists for historical compatibility and should not be used.
type VolumesPagedResponse legacyPagedResponse[Volume]

// Deprecated: VPCsPagedResponse exists for historical compatibility and should not be used.
type VPCsPagedResponse legacyPagedResponse[VPC]

// Deprecated: VPCSubnetsPagedResponse exists for historical compatibility and should not be used.
type VPCSubnetsPagedResponse legacyPagedResponse[VPCSubnet]
