// Code generated by protoc-gen-goext. DO NOT EDIT.

package dns

import (
	operation "github.com/yandex-cloud/go-genproto/yandex/cloud/operation"
	fieldmaskpb "google.golang.org/protobuf/types/known/fieldmaskpb"
)

func (m *GetDnsInboundEndpointRequest) SetDnsInboundEndpointId(v string) {
	m.DnsInboundEndpointId = v
}

func (m *CreateDnsInboundEndpointRequest) SetFolderId(v string) {
	m.FolderId = v
}

func (m *CreateDnsInboundEndpointRequest) SetName(v string) {
	m.Name = v
}

func (m *CreateDnsInboundEndpointRequest) SetDescription(v string) {
	m.Description = v
}

func (m *CreateDnsInboundEndpointRequest) SetLabels(v map[string]string) {
	m.Labels = v
}

func (m *CreateDnsInboundEndpointRequest) SetNetworkId(v string) {
	m.NetworkId = v
}

func (m *CreateDnsInboundEndpointRequest) SetDeletionProtection(v bool) {
	m.DeletionProtection = v
}

func (m *CreateDnsInboundEndpointRequest) SetAddressId(v string) {
	m.AddressId = v
}

func (m *CreateDnsInboundEndpointMetadata) SetDnsInboundEndpointId(v string) {
	m.DnsInboundEndpointId = v
}

func (m *UpdateDnsInboundEndpointRequest) SetDnsInboundEndpointId(v string) {
	m.DnsInboundEndpointId = v
}

func (m *UpdateDnsInboundEndpointRequest) SetUpdateMask(v *fieldmaskpb.FieldMask) {
	m.UpdateMask = v
}

func (m *UpdateDnsInboundEndpointRequest) SetFolderId(v string) {
	m.FolderId = v
}

func (m *UpdateDnsInboundEndpointRequest) SetName(v string) {
	m.Name = v
}

func (m *UpdateDnsInboundEndpointRequest) SetDescription(v string) {
	m.Description = v
}

func (m *UpdateDnsInboundEndpointRequest) SetLabels(v map[string]string) {
	m.Labels = v
}

func (m *UpdateDnsInboundEndpointRequest) SetDeletionProtection(v bool) {
	m.DeletionProtection = v
}

func (m *UpdateDnsInboundEndpointMetadata) SetDnsInboundEndpointId(v string) {
	m.DnsInboundEndpointId = v
}

func (m *DeleteDnsInboundEndpointRequest) SetDnsInboundEndpointId(v string) {
	m.DnsInboundEndpointId = v
}

func (m *DeleteDnsInboundEndpointMetadata) SetDnsInboundEndpointId(v string) {
	m.DnsInboundEndpointId = v
}

func (m *ListDnsInboundEndpointsRequest) SetFolderId(v string) {
	m.FolderId = v
}

func (m *ListDnsInboundEndpointsRequest) SetPageSize(v int64) {
	m.PageSize = v
}

func (m *ListDnsInboundEndpointsRequest) SetPageToken(v string) {
	m.PageToken = v
}

func (m *ListDnsInboundEndpointsRequest) SetFilter(v string) {
	m.Filter = v
}

func (m *ListDnsInboundEndpointsResponse) SetEndpoints(v []*DnsInboundEndpoint) {
	m.Endpoints = v
}

func (m *ListDnsInboundEndpointsResponse) SetNextPageToken(v string) {
	m.NextPageToken = v
}

func (m *ListDnsInboundEndpointOperationsRequest) SetDnsInboundEndpointId(v string) {
	m.DnsInboundEndpointId = v
}

func (m *ListDnsInboundEndpointOperationsRequest) SetPageSize(v int64) {
	m.PageSize = v
}

func (m *ListDnsInboundEndpointOperationsRequest) SetPageToken(v string) {
	m.PageToken = v
}

func (m *ListDnsInboundEndpointOperationsRequest) SetFilter(v string) {
	m.Filter = v
}

func (m *ListDnsInboundEndpointOperationsResponse) SetOperations(v []*operation.Operation) {
	m.Operations = v
}

func (m *ListDnsInboundEndpointOperationsResponse) SetNextPageToken(v string) {
	m.NextPageToken = v
}
