// Copyright 2022-2025 The sacloud/iaas-api-go Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package accessor

import (
	"net"

	"github.com/sacloud/packages-go/cidr"
)

// AssignedIPAddress スイッチ+ルータの割り当てられたIPアドレスリスト
type AssignedIPAddress interface {
	GetAssignedIPAddressMax() string
	SetAssignedIPAddressMax(v string)
	GetAssignedIPAddressMin() string
	SetAssignedIPAddressMin(v string)
}

// GetAssignedIPAddresses 最小/最大IPアドレスからIPアドレスリストを算出して返す
func GetAssignedIPAddresses(target AssignedIPAddress) []string {
	base := net.ParseIP(target.GetAssignedIPAddressMin())
	max := net.ParseIP(target.GetAssignedIPAddressMax())
	addresses := []string{base.String()}

	for {
		current := cidr.Inc(base)
		addresses = append(addresses, current.String())

		if current.Equal(max) {
			break
		}
		base = current
	}

	return addresses
}
