// Code generated by protoc-gen-go-grpc. DO NOT EDIT.
// versions:
// - protoc-gen-go-grpc v1.6.1
// - protoc             v3.21.12
// source: yandex/cloud/iam/v1/access_policy_template_service.proto

package iam

import (
	context "context"
	grpc "google.golang.org/grpc"
	codes "google.golang.org/grpc/codes"
	status "google.golang.org/grpc/status"
)

// This is a compile-time assertion to ensure that this generated file
// is compatible with the grpc package it is being compiled against.
// Requires gRPC-Go v1.64.0 or later.
const _ = grpc.SupportPackageIsVersion9

const (
	AccessPolicyTemplateService_List_FullMethodName = "/yandex.cloud.iam.v1.AccessPolicyTemplateService/List"
)

// AccessPolicyTemplateServiceClient is the client API for AccessPolicyTemplateService service.
//
// For semantics around ctx use and closing/ending streaming RPCs, please refer to https://pkg.go.dev/google.golang.org/grpc/?tab=doc#ClientConn.NewStream.
type AccessPolicyTemplateServiceClient interface {
	// Returns list of available access policy templates.
	List(ctx context.Context, in *ListAccessPolicyTemplatesRequest, opts ...grpc.CallOption) (*ListAccessPolicyTemplatesResponse, error)
}

type accessPolicyTemplateServiceClient struct {
	cc grpc.ClientConnInterface
}

func NewAccessPolicyTemplateServiceClient(cc grpc.ClientConnInterface) AccessPolicyTemplateServiceClient {
	return &accessPolicyTemplateServiceClient{cc}
}

func (c *accessPolicyTemplateServiceClient) List(ctx context.Context, in *ListAccessPolicyTemplatesRequest, opts ...grpc.CallOption) (*ListAccessPolicyTemplatesResponse, error) {
	cOpts := append([]grpc.CallOption{grpc.StaticMethod()}, opts...)
	out := new(ListAccessPolicyTemplatesResponse)
	err := c.cc.Invoke(ctx, AccessPolicyTemplateService_List_FullMethodName, in, out, cOpts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

// AccessPolicyTemplateServiceServer is the server API for AccessPolicyTemplateService service.
// All implementations should embed UnimplementedAccessPolicyTemplateServiceServer
// for forward compatibility.
type AccessPolicyTemplateServiceServer interface {
	// Returns list of available access policy templates.
	List(context.Context, *ListAccessPolicyTemplatesRequest) (*ListAccessPolicyTemplatesResponse, error)
}

// UnimplementedAccessPolicyTemplateServiceServer should be embedded to have
// forward compatible implementations.
//
// NOTE: this should be embedded by value instead of pointer to avoid a nil
// pointer dereference when methods are called.
type UnimplementedAccessPolicyTemplateServiceServer struct{}

func (UnimplementedAccessPolicyTemplateServiceServer) List(context.Context, *ListAccessPolicyTemplatesRequest) (*ListAccessPolicyTemplatesResponse, error) {
	return nil, status.Error(codes.Unimplemented, "method List not implemented")
}
func (UnimplementedAccessPolicyTemplateServiceServer) testEmbeddedByValue() {}

// UnsafeAccessPolicyTemplateServiceServer may be embedded to opt out of forward compatibility for this service.
// Use of this interface is not recommended, as added methods to AccessPolicyTemplateServiceServer will
// result in compilation errors.
type UnsafeAccessPolicyTemplateServiceServer interface {
	mustEmbedUnimplementedAccessPolicyTemplateServiceServer()
}

func RegisterAccessPolicyTemplateServiceServer(s grpc.ServiceRegistrar, srv AccessPolicyTemplateServiceServer) {
	// If the following call panics, it indicates UnimplementedAccessPolicyTemplateServiceServer was
	// embedded by pointer and is nil.  This will cause panics if an
	// unimplemented method is ever invoked, so we test this at initialization
	// time to prevent it from happening at runtime later due to I/O.
	if t, ok := srv.(interface{ testEmbeddedByValue() }); ok {
		t.testEmbeddedByValue()
	}
	s.RegisterService(&AccessPolicyTemplateService_ServiceDesc, srv)
}

func _AccessPolicyTemplateService_List_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(ListAccessPolicyTemplatesRequest)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(AccessPolicyTemplateServiceServer).List(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: AccessPolicyTemplateService_List_FullMethodName,
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(AccessPolicyTemplateServiceServer).List(ctx, req.(*ListAccessPolicyTemplatesRequest))
	}
	return interceptor(ctx, in, info, handler)
}

// AccessPolicyTemplateService_ServiceDesc is the grpc.ServiceDesc for AccessPolicyTemplateService service.
// It's only intended for direct use with grpc.RegisterService,
// and not to be introspected or modified (even as a copy)
var AccessPolicyTemplateService_ServiceDesc = grpc.ServiceDesc{
	ServiceName: "yandex.cloud.iam.v1.AccessPolicyTemplateService",
	HandlerType: (*AccessPolicyTemplateServiceServer)(nil),
	Methods: []grpc.MethodDesc{
		{
			MethodName: "List",
			Handler:    _AccessPolicyTemplateService_List_Handler,
		},
	},
	Streams:  []grpc.StreamDesc{},
	Metadata: "yandex/cloud/iam/v1/access_policy_template_service.proto",
}
