// Code generated by 'yaegi extract go/version'. DO NOT EDIT.

//go:build go1.22
// +build go1.22

package stdlib

import (
	"go/version"
	"reflect"
)

func init() {
	Symbols["go/version/version"] = map[string]reflect.Value{
		// function, constant and variable definitions
		"Compare": reflect.ValueOf(version.Compare),
		"IsValid": reflect.ValueOf(version.IsValid),
		"Lang":    reflect.ValueOf(version.Lang),
	}
}
