// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v3.21.12
// source: yandex/cloud/endpoint/api_endpoint_service.proto

package endpoint

import (
	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type GetApiEndpointRequest struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	ApiEndpointId string                 `protobuf:"bytes,1,opt,name=api_endpoint_id,json=apiEndpointId,proto3" json:"api_endpoint_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetApiEndpointRequest) Reset() {
	*x = GetApiEndpointRequest{}
	mi := &file_yandex_cloud_endpoint_api_endpoint_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetApiEndpointRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetApiEndpointRequest) ProtoMessage() {}

func (x *GetApiEndpointRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_endpoint_api_endpoint_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetApiEndpointRequest.ProtoReflect.Descriptor instead.
func (*GetApiEndpointRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_endpoint_api_endpoint_service_proto_rawDescGZIP(), []int{0}
}

func (x *GetApiEndpointRequest) GetApiEndpointId() string {
	if x != nil {
		return x.ApiEndpointId
	}
	return ""
}

type ListApiEndpointsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Deprecated: Marked as deprecated in yandex/cloud/endpoint/api_endpoint_service.proto.
	PageSize int64 `protobuf:"varint,1,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// Deprecated: Marked as deprecated in yandex/cloud/endpoint/api_endpoint_service.proto.
	PageToken     string `protobuf:"bytes,2,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListApiEndpointsRequest) Reset() {
	*x = ListApiEndpointsRequest{}
	mi := &file_yandex_cloud_endpoint_api_endpoint_service_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListApiEndpointsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListApiEndpointsRequest) ProtoMessage() {}

func (x *ListApiEndpointsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_endpoint_api_endpoint_service_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListApiEndpointsRequest.ProtoReflect.Descriptor instead.
func (*ListApiEndpointsRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_endpoint_api_endpoint_service_proto_rawDescGZIP(), []int{1}
}

// Deprecated: Marked as deprecated in yandex/cloud/endpoint/api_endpoint_service.proto.
func (x *ListApiEndpointsRequest) GetPageSize() int64 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

// Deprecated: Marked as deprecated in yandex/cloud/endpoint/api_endpoint_service.proto.
func (x *ListApiEndpointsRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

type ListApiEndpointsResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Endpoints     []*ApiEndpoint         `protobuf:"bytes,1,rep,name=endpoints,proto3" json:"endpoints,omitempty"`
	NextPageToken string                 `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListApiEndpointsResponse) Reset() {
	*x = ListApiEndpointsResponse{}
	mi := &file_yandex_cloud_endpoint_api_endpoint_service_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListApiEndpointsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListApiEndpointsResponse) ProtoMessage() {}

func (x *ListApiEndpointsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_endpoint_api_endpoint_service_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListApiEndpointsResponse.ProtoReflect.Descriptor instead.
func (*ListApiEndpointsResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_endpoint_api_endpoint_service_proto_rawDescGZIP(), []int{2}
}

func (x *ListApiEndpointsResponse) GetEndpoints() []*ApiEndpoint {
	if x != nil {
		return x.Endpoints
	}
	return nil
}

func (x *ListApiEndpointsResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

var File_yandex_cloud_endpoint_api_endpoint_service_proto protoreflect.FileDescriptor

const file_yandex_cloud_endpoint_api_endpoint_service_proto_rawDesc = "" +
	"\n" +
	"0yandex/cloud/endpoint/api_endpoint_service.proto\x12\x15yandex.cloud.endpoint\x1a\x1cgoogle/api/annotations.proto\x1a(yandex/cloud/endpoint/api_endpoint.proto\"?\n" +
	"\x15GetApiEndpointRequest\x12&\n" +
	"\x0fapi_endpoint_id\x18\x01 \x01(\tR\rapiEndpointId\"]\n" +
	"\x17ListApiEndpointsRequest\x12\x1f\n" +
	"\tpage_size\x18\x01 \x01(\x03B\x02\x18\x01R\bpageSize\x12!\n" +
	"\n" +
	"page_token\x18\x02 \x01(\tB\x02\x18\x01R\tpageToken\"\x84\x01\n" +
	"\x18ListApiEndpointsResponse\x12@\n" +
	"\tendpoints\x18\x01 \x03(\v2\".yandex.cloud.endpoint.ApiEndpointR\tendpoints\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\x90\x02\n" +
	"\x12ApiEndpointService\x12}\n" +
	"\x03Get\x12,.yandex.cloud.endpoint.GetApiEndpointRequest\x1a\".yandex.cloud.endpoint.ApiEndpoint\"$\x82\xd3\xe4\x93\x02\x1e\x12\x1c/endpoints/{api_endpoint_id}\x12{\n" +
	"\x04List\x12..yandex.cloud.endpoint.ListApiEndpointsRequest\x1a/.yandex.cloud.endpoint.ListApiEndpointsResponse\"\x12\x82\xd3\xe4\x93\x02\f\x12\n" +
	"/endpointsB_\n" +
	"\x19yandex.cloud.api.endpointZBgithub.com/yandex-cloud/go-genproto/yandex/cloud/endpoint;endpointb\x06proto3"

var (
	file_yandex_cloud_endpoint_api_endpoint_service_proto_rawDescOnce sync.Once
	file_yandex_cloud_endpoint_api_endpoint_service_proto_rawDescData []byte
)

func file_yandex_cloud_endpoint_api_endpoint_service_proto_rawDescGZIP() []byte {
	file_yandex_cloud_endpoint_api_endpoint_service_proto_rawDescOnce.Do(func() {
		file_yandex_cloud_endpoint_api_endpoint_service_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_yandex_cloud_endpoint_api_endpoint_service_proto_rawDesc), len(file_yandex_cloud_endpoint_api_endpoint_service_proto_rawDesc)))
	})
	return file_yandex_cloud_endpoint_api_endpoint_service_proto_rawDescData
}

var file_yandex_cloud_endpoint_api_endpoint_service_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_yandex_cloud_endpoint_api_endpoint_service_proto_goTypes = []any{
	(*GetApiEndpointRequest)(nil),    // 0: yandex.cloud.endpoint.GetApiEndpointRequest
	(*ListApiEndpointsRequest)(nil),  // 1: yandex.cloud.endpoint.ListApiEndpointsRequest
	(*ListApiEndpointsResponse)(nil), // 2: yandex.cloud.endpoint.ListApiEndpointsResponse
	(*ApiEndpoint)(nil),              // 3: yandex.cloud.endpoint.ApiEndpoint
}
var file_yandex_cloud_endpoint_api_endpoint_service_proto_depIdxs = []int32{
	3, // 0: yandex.cloud.endpoint.ListApiEndpointsResponse.endpoints:type_name -> yandex.cloud.endpoint.ApiEndpoint
	0, // 1: yandex.cloud.endpoint.ApiEndpointService.Get:input_type -> yandex.cloud.endpoint.GetApiEndpointRequest
	1, // 2: yandex.cloud.endpoint.ApiEndpointService.List:input_type -> yandex.cloud.endpoint.ListApiEndpointsRequest
	3, // 3: yandex.cloud.endpoint.ApiEndpointService.Get:output_type -> yandex.cloud.endpoint.ApiEndpoint
	2, // 4: yandex.cloud.endpoint.ApiEndpointService.List:output_type -> yandex.cloud.endpoint.ListApiEndpointsResponse
	3, // [3:5] is the sub-list for method output_type
	1, // [1:3] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_yandex_cloud_endpoint_api_endpoint_service_proto_init() }
func file_yandex_cloud_endpoint_api_endpoint_service_proto_init() {
	if File_yandex_cloud_endpoint_api_endpoint_service_proto != nil {
		return
	}
	file_yandex_cloud_endpoint_api_endpoint_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_yandex_cloud_endpoint_api_endpoint_service_proto_rawDesc), len(file_yandex_cloud_endpoint_api_endpoint_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_yandex_cloud_endpoint_api_endpoint_service_proto_goTypes,
		DependencyIndexes: file_yandex_cloud_endpoint_api_endpoint_service_proto_depIdxs,
		MessageInfos:      file_yandex_cloud_endpoint_api_endpoint_service_proto_msgTypes,
	}.Build()
	File_yandex_cloud_endpoint_api_endpoint_service_proto = out.File
	file_yandex_cloud_endpoint_api_endpoint_service_proto_goTypes = nil
	file_yandex_cloud_endpoint_api_endpoint_service_proto_depIdxs = nil
}
