// Code generated by protoc-gen-goext. DO NOT EDIT.

package dns

import (
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
)

func (m *DnsInboundEndpoint) SetId(v string) {
	m.Id = v
}

func (m *DnsInboundEndpoint) SetFolderId(v string) {
	m.FolderId = v
}

func (m *DnsInboundEndpoint) SetCreatedAt(v *timestamppb.Timestamp) {
	m.CreatedAt = v
}

func (m *DnsInboundEndpoint) SetName(v string) {
	m.Name = v
}

func (m *DnsInboundEndpoint) SetDescription(v string) {
	m.Description = v
}

func (m *DnsInboundEndpoint) SetLabels(v map[string]string) {
	m.Labels = v
}

func (m *DnsInboundEndpoint) SetNetworkId(v string) {
	m.NetworkId = v
}

func (m *DnsInboundEndpoint) SetAddress(v string) {
	m.Address = v
}

func (m *DnsInboundEndpoint) SetAddressId(v string) {
	m.AddressId = v
}

func (m *DnsInboundEndpoint) SetStatus(v DnsInboundEndpoint_Status) {
	m.Status = v
}

func (m *DnsInboundEndpoint) SetDeletionProtection(v bool) {
	m.DeletionProtection = v
}
