/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by lister-gen. DO NOT EDIT.

package v1

// BackendTLSPolicyListerExpansion allows custom methods to be added to
// BackendTLSPolicyLister.
type BackendTLSPolicyListerExpansion interface{}

// BackendTLSPolicyNamespaceListerExpansion allows custom methods to be added to
// BackendTLSPolicyNamespaceLister.
type BackendTLSPolicyNamespaceListerExpansion interface{}

// GRPCRouteListerExpansion allows custom methods to be added to
// GRPCRouteLister.
type GRPCRouteListerExpansion interface{}

// GRPCRouteNamespaceListerExpansion allows custom methods to be added to
// GRPCRouteNamespaceLister.
type GRPCRouteNamespaceListerExpansion interface{}

// GatewayListerExpansion allows custom methods to be added to
// GatewayLister.
type GatewayListerExpansion interface{}

// GatewayNamespaceListerExpansion allows custom methods to be added to
// GatewayNamespaceLister.
type GatewayNamespaceListerExpansion interface{}

// GatewayClassListerExpansion allows custom methods to be added to
// GatewayClassLister.
type GatewayClassListerExpansion interface{}

// HTTPRouteListerExpansion allows custom methods to be added to
// HTTPRouteLister.
type HTTPRouteListerExpansion interface{}

// HTTPRouteNamespaceListerExpansion allows custom methods to be added to
// HTTPRouteNamespaceLister.
type HTTPRouteNamespaceListerExpansion interface{}

// ListenerSetListerExpansion allows custom methods to be added to
// ListenerSetLister.
type ListenerSetListerExpansion interface{}

// ListenerSetNamespaceListerExpansion allows custom methods to be added to
// ListenerSetNamespaceLister.
type ListenerSetNamespaceListerExpansion interface{}

// ReferenceGrantListerExpansion allows custom methods to be added to
// ReferenceGrantLister.
type ReferenceGrantListerExpansion interface{}

// ReferenceGrantNamespaceListerExpansion allows custom methods to be added to
// ReferenceGrantNamespaceLister.
type ReferenceGrantNamespaceListerExpansion interface{}

// TLSRouteListerExpansion allows custom methods to be added to
// TLSRouteLister.
type TLSRouteListerExpansion interface{}

// TLSRouteNamespaceListerExpansion allows custom methods to be added to
// TLSRouteNamespaceLister.
type TLSRouteNamespaceListerExpansion interface{}
