// Code generated by protoc-gen-goext. DO NOT EDIT.

package iam

func (m *GetServiceRequest) SetServiceId(v string) {
	m.ServiceId = v
}

func (m *GetServiceRequest) SetResource(v *Resource) {
	m.Resource = v
}

func (m *ListServicesRequest) SetResource(v *Resource) {
	m.Resource = v
}

func (m *ListServicesRequest) SetPageSize(v int64) {
	m.PageSize = v
}

func (m *ListServicesRequest) SetPageToken(v string) {
	m.PageToken = v
}

func (m *ListServicesResponse) SetServices(v []*Service) {
	m.Services = v
}

func (m *ListServicesResponse) SetNextPageToken(v string) {
	m.NextPageToken = v
}

func (m *EnableServiceRequest) SetServiceId(v string) {
	m.ServiceId = v
}

func (m *EnableServiceRequest) SetResource(v *Resource) {
	m.Resource = v
}

func (m *EnableServiceMetadata) SetServiceId(v string) {
	m.ServiceId = v
}

func (m *EnableServiceMetadata) SetResource(v *Resource) {
	m.Resource = v
}

func (m *DisableServiceRequest) SetServiceId(v string) {
	m.ServiceId = v
}

func (m *DisableServiceRequest) SetResource(v *Resource) {
	m.Resource = v
}

func (m *DisableServiceMetadata) SetServiceId(v string) {
	m.ServiceId = v
}

func (m *DisableServiceMetadata) SetResource(v *Resource) {
	m.Resource = v
}

func (m *ResolveServiceAgentRequest) SetServiceId(v string) {
	m.ServiceId = v
}

func (m *ResolveServiceAgentRequest) SetMicroserviceId(v string) {
	m.MicroserviceId = v
}

func (m *ResolveServiceAgentRequest) SetResource(v *Resource) {
	m.Resource = v
}
