/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

import (
	apisv1 "sigs.k8s.io/gateway-api/apis/v1"
)

// ReferenceGrantToApplyConfiguration represents a declarative configuration of the ReferenceGrantTo type for use
// with apply.
//
// ReferenceGrantTo describes what Kinds are allowed as targets of the
// references.
type ReferenceGrantToApplyConfiguration struct {
	// Group is the group of the referent.
	// When empty, the Kubernetes core API group is inferred.
	//
	// Support: Core
	Group *apisv1.Group `json:"group,omitempty"`
	// Kind is the kind of the referent. Although implementations may support
	// additional resources, the following types are part of the "Core"
	// support level for this field:
	//
	// * Secret when used to permit a SecretObjectReference
	// * Service when used to permit a BackendObjectReference
	Kind *apisv1.Kind `json:"kind,omitempty"`
	// Name is the name of the referent. When unspecified, this policy
	// refers to all resources of the specified Group and Kind in the local
	// namespace.
	Name *apisv1.ObjectName `json:"name,omitempty"`
}

// ReferenceGrantToApplyConfiguration constructs a declarative configuration of the ReferenceGrantTo type for use with
// apply.
func ReferenceGrantTo() *ReferenceGrantToApplyConfiguration {
	return &ReferenceGrantToApplyConfiguration{}
}

// WithGroup sets the Group field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Group field is set to the value of the last call.
func (b *ReferenceGrantToApplyConfiguration) WithGroup(value apisv1.Group) *ReferenceGrantToApplyConfiguration {
	b.Group = &value
	return b
}

// WithKind sets the Kind field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Kind field is set to the value of the last call.
func (b *ReferenceGrantToApplyConfiguration) WithKind(value apisv1.Kind) *ReferenceGrantToApplyConfiguration {
	b.Kind = &value
	return b
}

// WithName sets the Name field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Name field is set to the value of the last call.
func (b *ReferenceGrantToApplyConfiguration) WithName(value apisv1.ObjectName) *ReferenceGrantToApplyConfiguration {
	b.Name = &value
	return b
}
