/*
Copyright 2018 Comcast Cable Communications Management, LLC
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package vinyldns

// recordSetsList retrieves the list of record sets with the List criteria passed,
// for the specified zone.
func (c *Client) recordSetsList(zoneID string, filter ListFilter) (*RecordSetsResponse, error) {
	recordSets := &RecordSetsResponse{}
	err := resourceRequest(c, recordSetsListEP(c, zoneID, filter), "GET", nil, recordSets)
	if err != nil {
		return recordSets, err
	}

	return recordSets, nil
}

// recordSetsGlobalList retrieves the list of record sets with the List criteria passed,
// across all zones.
func (c *Client) recordSetsGlobalList(filter GlobalListFilter) (*RecordSetsResponse, error) {
	recordSets := &RecordSetsResponse{}
	err := resourceRequest(c, recordSetsGlobalListEP(c, filter), "GET", nil, recordSets)
	if err != nil {
		return recordSets, err
	}

	return recordSets, nil
}
