// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v3.21.12
// source: yandex/cloud/iam/v1/package_options.proto

package iam

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

var File_yandex_cloud_iam_v1_package_options_proto protoreflect.FileDescriptor

const file_yandex_cloud_iam_v1_package_options_proto_rawDesc = "" +
	"\n" +
	")yandex/cloud/iam/v1/package_options.proto\x12\x13yandex.cloud.iam.v1B=Z;github.com/yandex-cloud/go-genproto/yandex/cloud/iam/v1;iamb\x06proto3"

var file_yandex_cloud_iam_v1_package_options_proto_goTypes = []any{}
var file_yandex_cloud_iam_v1_package_options_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_yandex_cloud_iam_v1_package_options_proto_init() }
func file_yandex_cloud_iam_v1_package_options_proto_init() {
	if File_yandex_cloud_iam_v1_package_options_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_package_options_proto_rawDesc), len(file_yandex_cloud_iam_v1_package_options_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   0,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_yandex_cloud_iam_v1_package_options_proto_goTypes,
		DependencyIndexes: file_yandex_cloud_iam_v1_package_options_proto_depIdxs,
	}.Build()
	File_yandex_cloud_iam_v1_package_options_proto = out.File
	file_yandex_cloud_iam_v1_package_options_proto_goTypes = nil
	file_yandex_cloud_iam_v1_package_options_proto_depIdxs = nil
}
