// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v3.21.12
// source: yandex/cloud/iam/v1/role_service.proto

package iam

import (
	_ "github.com/yandex-cloud/go-genproto/yandex/cloud"
	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type GetRoleRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the Role resource to return.
	// To get the role ID, use a [RoleService.List] request.
	RoleId        string `protobuf:"bytes,1,opt,name=role_id,json=roleId,proto3" json:"role_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetRoleRequest) Reset() {
	*x = GetRoleRequest{}
	mi := &file_yandex_cloud_iam_v1_role_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetRoleRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetRoleRequest) ProtoMessage() {}

func (x *GetRoleRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_role_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetRoleRequest.ProtoReflect.Descriptor instead.
func (*GetRoleRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_role_service_proto_rawDescGZIP(), []int{0}
}

func (x *GetRoleRequest) GetRoleId() string {
	if x != nil {
		return x.RoleId
	}
	return ""
}

type ListRolesRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The maximum number of results per page to return. If the number of available
	// results is larger than [page_size],
	// the service returns a [ListRolesResponse.next_page_token]
	// that can be used to get the next page of results in subsequent list requests.
	// Default value: 100.
	PageSize int64 `protobuf:"varint,1,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// Page token. To get the next page of results, set [page_token]
	// to the [ListRolesResponse.next_page_token]
	// returned by a previous list request.
	PageToken string `protobuf:"bytes,2,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	// A filter expression that filters resources listed in the response.
	Filter        string `protobuf:"bytes,3,opt,name=filter,proto3" json:"filter,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListRolesRequest) Reset() {
	*x = ListRolesRequest{}
	mi := &file_yandex_cloud_iam_v1_role_service_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListRolesRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListRolesRequest) ProtoMessage() {}

func (x *ListRolesRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_role_service_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListRolesRequest.ProtoReflect.Descriptor instead.
func (*ListRolesRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_role_service_proto_rawDescGZIP(), []int{1}
}

func (x *ListRolesRequest) GetPageSize() int64 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListRolesRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

func (x *ListRolesRequest) GetFilter() string {
	if x != nil {
		return x.Filter
	}
	return ""
}

type ListRolesResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// List of Role resources.
	Roles []*Role `protobuf:"bytes,1,rep,name=roles,proto3" json:"roles,omitempty"`
	// This token allows you to get the next page of results for list requests. If the number of results
	// is larger than [ListRolesRequest.page_size], use
	// the [next_page_token] as the value
	// for the [ListRolesRequest.page_token] query parameter
	// in the next list request. Each subsequent list request will have its own
	// [next_page_token] to continue paging through the results.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListRolesResponse) Reset() {
	*x = ListRolesResponse{}
	mi := &file_yandex_cloud_iam_v1_role_service_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListRolesResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListRolesResponse) ProtoMessage() {}

func (x *ListRolesResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_role_service_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListRolesResponse.ProtoReflect.Descriptor instead.
func (*ListRolesResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_role_service_proto_rawDescGZIP(), []int{2}
}

func (x *ListRolesResponse) GetRoles() []*Role {
	if x != nil {
		return x.Roles
	}
	return nil
}

func (x *ListRolesResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

var File_yandex_cloud_iam_v1_role_service_proto protoreflect.FileDescriptor

const file_yandex_cloud_iam_v1_role_service_proto_rawDesc = "" +
	"\n" +
	"&yandex/cloud/iam/v1/role_service.proto\x12\x13yandex.cloud.iam.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x1dyandex/cloud/validation.proto\x1a\x1eyandex/cloud/iam/v1/role.proto\"7\n" +
	"\x0eGetRoleRequest\x12%\n" +
	"\arole_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\x06roleId\"\x8a\x01\n" +
	"\x10ListRolesRequest\x12'\n" +
	"\tpage_size\x18\x01 \x01(\x03B\n" +
	"\xfa\xc71\x06<=1000R\bpageSize\x12)\n" +
	"\n" +
	"page_token\x18\x02 \x01(\tB\n" +
	"\x8a\xc81\x06<=2000R\tpageToken\x12\"\n" +
	"\x06filter\x18\x03 \x01(\tB\n" +
	"\x8a\xc81\x06<=1000R\x06filter\"l\n" +
	"\x11ListRolesResponse\x12/\n" +
	"\x05roles\x18\x01 \x03(\v2\x19.yandex.cloud.iam.v1.RoleR\x05roles\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\xe3\x01\n" +
	"\vRoleService\x12f\n" +
	"\x03Get\x12#.yandex.cloud.iam.v1.GetRoleRequest\x1a\x19.yandex.cloud.iam.v1.Role\"\x1f\x82\xd3\xe4\x93\x02\x19\x12\x17/iam/v1/roles/{role_id}\x12l\n" +
	"\x04List\x12%.yandex.cloud.iam.v1.ListRolesRequest\x1a&.yandex.cloud.iam.v1.ListRolesResponse\"\x15\x82\xd3\xe4\x93\x02\x0f\x12\r/iam/v1/rolesBV\n" +
	"\x17yandex.cloud.api.iam.v1Z;github.com/yandex-cloud/go-genproto/yandex/cloud/iam/v1;iamb\x06proto3"

var (
	file_yandex_cloud_iam_v1_role_service_proto_rawDescOnce sync.Once
	file_yandex_cloud_iam_v1_role_service_proto_rawDescData []byte
)

func file_yandex_cloud_iam_v1_role_service_proto_rawDescGZIP() []byte {
	file_yandex_cloud_iam_v1_role_service_proto_rawDescOnce.Do(func() {
		file_yandex_cloud_iam_v1_role_service_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_role_service_proto_rawDesc), len(file_yandex_cloud_iam_v1_role_service_proto_rawDesc)))
	})
	return file_yandex_cloud_iam_v1_role_service_proto_rawDescData
}

var file_yandex_cloud_iam_v1_role_service_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_yandex_cloud_iam_v1_role_service_proto_goTypes = []any{
	(*GetRoleRequest)(nil),    // 0: yandex.cloud.iam.v1.GetRoleRequest
	(*ListRolesRequest)(nil),  // 1: yandex.cloud.iam.v1.ListRolesRequest
	(*ListRolesResponse)(nil), // 2: yandex.cloud.iam.v1.ListRolesResponse
	(*Role)(nil),              // 3: yandex.cloud.iam.v1.Role
}
var file_yandex_cloud_iam_v1_role_service_proto_depIdxs = []int32{
	3, // 0: yandex.cloud.iam.v1.ListRolesResponse.roles:type_name -> yandex.cloud.iam.v1.Role
	0, // 1: yandex.cloud.iam.v1.RoleService.Get:input_type -> yandex.cloud.iam.v1.GetRoleRequest
	1, // 2: yandex.cloud.iam.v1.RoleService.List:input_type -> yandex.cloud.iam.v1.ListRolesRequest
	3, // 3: yandex.cloud.iam.v1.RoleService.Get:output_type -> yandex.cloud.iam.v1.Role
	2, // 4: yandex.cloud.iam.v1.RoleService.List:output_type -> yandex.cloud.iam.v1.ListRolesResponse
	3, // [3:5] is the sub-list for method output_type
	1, // [1:3] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_yandex_cloud_iam_v1_role_service_proto_init() }
func file_yandex_cloud_iam_v1_role_service_proto_init() {
	if File_yandex_cloud_iam_v1_role_service_proto != nil {
		return
	}
	file_yandex_cloud_iam_v1_role_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_role_service_proto_rawDesc), len(file_yandex_cloud_iam_v1_role_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_yandex_cloud_iam_v1_role_service_proto_goTypes,
		DependencyIndexes: file_yandex_cloud_iam_v1_role_service_proto_depIdxs,
		MessageInfos:      file_yandex_cloud_iam_v1_role_service_proto_msgTypes,
	}.Build()
	File_yandex_cloud_iam_v1_role_service_proto = out.File
	file_yandex_cloud_iam_v1_role_service_proto_goTypes = nil
	file_yandex_cloud_iam_v1_role_service_proto_depIdxs = nil
}
