// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v3.21.12
// source: yandex/cloud/iam/v1/refresh_token.proto

package iam

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Protection level of the refresh token.
type RefreshToken_ProtectionLevel int32

const (
	RefreshToken_PROTECTION_LEVEL_UNSPECIFIED RefreshToken_ProtectionLevel = 0
	// Refresh token without DPOP
	RefreshToken_NO_PROTECTION RefreshToken_ProtectionLevel = 1
	// Refresh token with dpop. The dpop key is not a YubiKey PIV key with required pin/touch policy and attestation.
	RefreshToken_INSECURE_KEY_DPOP RefreshToken_ProtectionLevel = 2
	// Refresh token with dpop. The dpop key is a YubiKey PIV key with required pin/touch policy and attestation.
	RefreshToken_SECURE_KEY_DPOP RefreshToken_ProtectionLevel = 3
)

// Enum value maps for RefreshToken_ProtectionLevel.
var (
	RefreshToken_ProtectionLevel_name = map[int32]string{
		0: "PROTECTION_LEVEL_UNSPECIFIED",
		1: "NO_PROTECTION",
		2: "INSECURE_KEY_DPOP",
		3: "SECURE_KEY_DPOP",
	}
	RefreshToken_ProtectionLevel_value = map[string]int32{
		"PROTECTION_LEVEL_UNSPECIFIED": 0,
		"NO_PROTECTION":                1,
		"INSECURE_KEY_DPOP":            2,
		"SECURE_KEY_DPOP":              3,
	}
)

func (x RefreshToken_ProtectionLevel) Enum() *RefreshToken_ProtectionLevel {
	p := new(RefreshToken_ProtectionLevel)
	*p = x
	return p
}

func (x RefreshToken_ProtectionLevel) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (RefreshToken_ProtectionLevel) Descriptor() protoreflect.EnumDescriptor {
	return file_yandex_cloud_iam_v1_refresh_token_proto_enumTypes[0].Descriptor()
}

func (RefreshToken_ProtectionLevel) Type() protoreflect.EnumType {
	return &file_yandex_cloud_iam_v1_refresh_token_proto_enumTypes[0]
}

func (x RefreshToken_ProtectionLevel) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use RefreshToken_ProtectionLevel.Descriptor instead.
func (RefreshToken_ProtectionLevel) EnumDescriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_refresh_token_proto_rawDescGZIP(), []int{0, 0}
}

type RefreshToken struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Refresh Token id.
	Id string `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	// Information about the app for which the Refresh Token was issued.
	ClientInstanceInfo string `protobuf:"bytes,2,opt,name=client_instance_info,json=clientInstanceInfo,proto3" json:"client_instance_info,omitempty"`
	// The OAuth client identifier for which the Refresh Token was issued.
	ClientId string `protobuf:"bytes,3,opt,name=client_id,json=clientId,proto3" json:"client_id,omitempty"`
	// The subject identifier for whom the Refresh Token was issued.
	SubjectId string `protobuf:"bytes,4,opt,name=subject_id,json=subjectId,proto3" json:"subject_id,omitempty"`
	// Refresh token creation time.
	CreatedAt *timestamppb.Timestamp `protobuf:"bytes,5,opt,name=created_at,json=createdAt,proto3" json:"created_at,omitempty"`
	// Refresh token expiration time.
	ExpiresAt *timestamppb.Timestamp `protobuf:"bytes,6,opt,name=expires_at,json=expiresAt,proto3" json:"expires_at,omitempty"`
	// Timestamp for the last authentication using this Refresh Token.
	LastUsedAt *timestamppb.Timestamp `protobuf:"bytes,7,opt,name=last_used_at,json=lastUsedAt,proto3" json:"last_used_at,omitempty"`
	// Protection level of the refresh token.
	//
	// It shows whether DPOP was used to protect the Refresh Token and and the level of security of the storage used for the DPOP key.
	ProtectionLevel RefreshToken_ProtectionLevel `protobuf:"varint,8,opt,name=protection_level,json=protectionLevel,proto3,enum=yandex.cloud.iam.v1.RefreshToken_ProtectionLevel" json:"protection_level,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *RefreshToken) Reset() {
	*x = RefreshToken{}
	mi := &file_yandex_cloud_iam_v1_refresh_token_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RefreshToken) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RefreshToken) ProtoMessage() {}

func (x *RefreshToken) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_refresh_token_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RefreshToken.ProtoReflect.Descriptor instead.
func (*RefreshToken) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_refresh_token_proto_rawDescGZIP(), []int{0}
}

func (x *RefreshToken) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

func (x *RefreshToken) GetClientInstanceInfo() string {
	if x != nil {
		return x.ClientInstanceInfo
	}
	return ""
}

func (x *RefreshToken) GetClientId() string {
	if x != nil {
		return x.ClientId
	}
	return ""
}

func (x *RefreshToken) GetSubjectId() string {
	if x != nil {
		return x.SubjectId
	}
	return ""
}

func (x *RefreshToken) GetCreatedAt() *timestamppb.Timestamp {
	if x != nil {
		return x.CreatedAt
	}
	return nil
}

func (x *RefreshToken) GetExpiresAt() *timestamppb.Timestamp {
	if x != nil {
		return x.ExpiresAt
	}
	return nil
}

func (x *RefreshToken) GetLastUsedAt() *timestamppb.Timestamp {
	if x != nil {
		return x.LastUsedAt
	}
	return nil
}

func (x *RefreshToken) GetProtectionLevel() RefreshToken_ProtectionLevel {
	if x != nil {
		return x.ProtectionLevel
	}
	return RefreshToken_PROTECTION_LEVEL_UNSPECIFIED
}

var File_yandex_cloud_iam_v1_refresh_token_proto protoreflect.FileDescriptor

const file_yandex_cloud_iam_v1_refresh_token_proto_rawDesc = "" +
	"\n" +
	"'yandex/cloud/iam/v1/refresh_token.proto\x12\x13yandex.cloud.iam.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"\x92\x04\n" +
	"\fRefreshToken\x12\x0e\n" +
	"\x02id\x18\x01 \x01(\tR\x02id\x120\n" +
	"\x14client_instance_info\x18\x02 \x01(\tR\x12clientInstanceInfo\x12\x1b\n" +
	"\tclient_id\x18\x03 \x01(\tR\bclientId\x12\x1d\n" +
	"\n" +
	"subject_id\x18\x04 \x01(\tR\tsubjectId\x129\n" +
	"\n" +
	"created_at\x18\x05 \x01(\v2\x1a.google.protobuf.TimestampR\tcreatedAt\x129\n" +
	"\n" +
	"expires_at\x18\x06 \x01(\v2\x1a.google.protobuf.TimestampR\texpiresAt\x12<\n" +
	"\flast_used_at\x18\a \x01(\v2\x1a.google.protobuf.TimestampR\n" +
	"lastUsedAt\x12\\\n" +
	"\x10protection_level\x18\b \x01(\x0e21.yandex.cloud.iam.v1.RefreshToken.ProtectionLevelR\x0fprotectionLevel\"r\n" +
	"\x0fProtectionLevel\x12 \n" +
	"\x1cPROTECTION_LEVEL_UNSPECIFIED\x10\x00\x12\x11\n" +
	"\rNO_PROTECTION\x10\x01\x12\x15\n" +
	"\x11INSECURE_KEY_DPOP\x10\x02\x12\x13\n" +
	"\x0fSECURE_KEY_DPOP\x10\x03BV\n" +
	"\x17yandex.cloud.api.iam.v1Z;github.com/yandex-cloud/go-genproto/yandex/cloud/iam/v1;iamb\x06proto3"

var (
	file_yandex_cloud_iam_v1_refresh_token_proto_rawDescOnce sync.Once
	file_yandex_cloud_iam_v1_refresh_token_proto_rawDescData []byte
)

func file_yandex_cloud_iam_v1_refresh_token_proto_rawDescGZIP() []byte {
	file_yandex_cloud_iam_v1_refresh_token_proto_rawDescOnce.Do(func() {
		file_yandex_cloud_iam_v1_refresh_token_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_refresh_token_proto_rawDesc), len(file_yandex_cloud_iam_v1_refresh_token_proto_rawDesc)))
	})
	return file_yandex_cloud_iam_v1_refresh_token_proto_rawDescData
}

var file_yandex_cloud_iam_v1_refresh_token_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_yandex_cloud_iam_v1_refresh_token_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_yandex_cloud_iam_v1_refresh_token_proto_goTypes = []any{
	(RefreshToken_ProtectionLevel)(0), // 0: yandex.cloud.iam.v1.RefreshToken.ProtectionLevel
	(*RefreshToken)(nil),              // 1: yandex.cloud.iam.v1.RefreshToken
	(*timestamppb.Timestamp)(nil),     // 2: google.protobuf.Timestamp
}
var file_yandex_cloud_iam_v1_refresh_token_proto_depIdxs = []int32{
	2, // 0: yandex.cloud.iam.v1.RefreshToken.created_at:type_name -> google.protobuf.Timestamp
	2, // 1: yandex.cloud.iam.v1.RefreshToken.expires_at:type_name -> google.protobuf.Timestamp
	2, // 2: yandex.cloud.iam.v1.RefreshToken.last_used_at:type_name -> google.protobuf.Timestamp
	0, // 3: yandex.cloud.iam.v1.RefreshToken.protection_level:type_name -> yandex.cloud.iam.v1.RefreshToken.ProtectionLevel
	4, // [4:4] is the sub-list for method output_type
	4, // [4:4] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	4, // [4:4] is the sub-list for extension extendee
	0, // [0:4] is the sub-list for field type_name
}

func init() { file_yandex_cloud_iam_v1_refresh_token_proto_init() }
func file_yandex_cloud_iam_v1_refresh_token_proto_init() {
	if File_yandex_cloud_iam_v1_refresh_token_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_refresh_token_proto_rawDesc), len(file_yandex_cloud_iam_v1_refresh_token_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_yandex_cloud_iam_v1_refresh_token_proto_goTypes,
		DependencyIndexes: file_yandex_cloud_iam_v1_refresh_token_proto_depIdxs,
		EnumInfos:         file_yandex_cloud_iam_v1_refresh_token_proto_enumTypes,
		MessageInfos:      file_yandex_cloud_iam_v1_refresh_token_proto_msgTypes,
	}.Build()
	File_yandex_cloud_iam_v1_refresh_token_proto = out.File
	file_yandex_cloud_iam_v1_refresh_token_proto_goTypes = nil
	file_yandex_cloud_iam_v1_refresh_token_proto_depIdxs = nil
}
