package bunny

// StorageZoneService communicates with the /storagezone API endpoint.
//
// Bunny.net API docs: https://docs.bunny.net/reference/storagezonepublic_index
type StorageZoneService struct {
	client *Client
}
