/*
Copyright 2020 The Knative Authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by lister-gen. DO NOT EDIT.

package v1alpha1

// CertificateListerExpansion allows custom methods to be added to
// CertificateLister.
type CertificateListerExpansion interface{}

// CertificateNamespaceListerExpansion allows custom methods to be added to
// CertificateNamespaceLister.
type CertificateNamespaceListerExpansion interface{}

// ClusterDomainClaimListerExpansion allows custom methods to be added to
// ClusterDomainClaimLister.
type ClusterDomainClaimListerExpansion interface{}

// IngressListerExpansion allows custom methods to be added to
// IngressLister.
type IngressListerExpansion interface{}

// IngressNamespaceListerExpansion allows custom methods to be added to
// IngressNamespaceLister.
type IngressNamespaceListerExpansion interface{}

// ServerlessServiceListerExpansion allows custom methods to be added to
// ServerlessServiceLister.
type ServerlessServiceListerExpansion interface{}

// ServerlessServiceNamespaceListerExpansion allows custom methods to be added to
// ServerlessServiceNamespaceLister.
type ServerlessServiceNamespaceListerExpansion interface{}
