/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

// GatewayTLSConfigApplyConfiguration represents a declarative configuration of the GatewayTLSConfig type for use
// with apply.
//
// GatewayTLSConfig specifies frontend and backend tls configuration for gateway.
type GatewayTLSConfigApplyConfiguration struct {
	// Backend describes TLS configuration for gateway when connecting
	// to backends.
	//
	// Note that this contains only details for the Gateway as a TLS client,
	// and does _not_ imply behavior about how to choose which backend should
	// get a TLS connection. That is determined by the presence of a BackendTLSPolicy.
	//
	// Support: Core
	Backend *GatewayBackendTLSApplyConfiguration `json:"backend,omitempty"`
	// Frontend describes TLS config when client connects to Gateway.
	// Support: Core
	Frontend *FrontendTLSConfigApplyConfiguration `json:"frontend,omitempty"`
}

// GatewayTLSConfigApplyConfiguration constructs a declarative configuration of the GatewayTLSConfig type for use with
// apply.
func GatewayTLSConfig() *GatewayTLSConfigApplyConfiguration {
	return &GatewayTLSConfigApplyConfiguration{}
}

// WithBackend sets the Backend field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Backend field is set to the value of the last call.
func (b *GatewayTLSConfigApplyConfiguration) WithBackend(value *GatewayBackendTLSApplyConfiguration) *GatewayTLSConfigApplyConfiguration {
	b.Backend = value
	return b
}

// WithFrontend sets the Frontend field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Frontend field is set to the value of the last call.
func (b *GatewayTLSConfigApplyConfiguration) WithFrontend(value *FrontendTLSConfigApplyConfiguration) *GatewayTLSConfigApplyConfiguration {
	b.Frontend = value
	return b
}
