/*
The MIT License (MIT)

Copyright (c) 2016-2020 Containous SAS; 2020-2026 Traefik Labs

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1alpha1

import (
	dynamic "github.com/traefik/traefik/v3/pkg/config/dynamic"
	intstr "k8s.io/apimachinery/pkg/util/intstr"
)

// MirroringApplyConfiguration represents a declarative configuration of the Mirroring type for use
// with apply.
//
// Mirroring holds the mirroring service configuration.
// More info: https://doc.traefik.io/traefik/v3.7/reference/routing-configuration/http/load-balancing/service/#mirroring
type MirroringApplyConfiguration struct {
	LoadBalancerSpecApplyConfiguration `json:",inline"`
	// MirrorBody defines whether the body of the request should be mirrored.
	// Default value is true.
	MirrorBody *bool `json:"mirrorBody,omitempty"`
	// MaxBodySize defines the maximum size allowed for the body of the request.
	// If the body is larger, the request is not mirrored.
	// Default value is -1, which means unlimited size.
	MaxBodySize *int64 `json:"maxBodySize,omitempty"`
	// Mirrors defines the list of mirrors where Traefik will duplicate the traffic.
	Mirrors []MirrorServiceApplyConfiguration `json:"mirrors,omitempty"`
}

// MirroringApplyConfiguration constructs a declarative configuration of the Mirroring type for use with
// apply.
func Mirroring() *MirroringApplyConfiguration {
	return &MirroringApplyConfiguration{}
}

// WithName sets the Name field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Name field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithName(value string) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.Name = &value
	return b
}

// WithKind sets the Kind field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Kind field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithKind(value string) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.Kind = &value
	return b
}

// WithNamespace sets the Namespace field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Namespace field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithNamespace(value string) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.Namespace = &value
	return b
}

// WithMiddlewares adds the given value to the Middlewares field in the declarative configuration
// and returns the receiver, so that objects can be build by chaining "With" function invocations.
// If called multiple times, values provided by each call will be appended to the Middlewares field.
func (b *MirroringApplyConfiguration) WithMiddlewares(values ...*MiddlewareRefApplyConfiguration) *MirroringApplyConfiguration {
	for i := range values {
		if values[i] == nil {
			panic("nil value passed to WithMiddlewares")
		}
		b.LoadBalancerSpecApplyConfiguration.Middlewares = append(b.LoadBalancerSpecApplyConfiguration.Middlewares, *values[i])
	}
	return b
}

// WithSticky sets the Sticky field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Sticky field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithSticky(value dynamic.Sticky) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.Sticky = &value
	return b
}

// WithPort sets the Port field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Port field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithPort(value intstr.IntOrString) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.Port = &value
	return b
}

// WithScheme sets the Scheme field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Scheme field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithScheme(value string) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.Scheme = &value
	return b
}

// WithStrategy sets the Strategy field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Strategy field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithStrategy(value dynamic.BalancerStrategy) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.Strategy = &value
	return b
}

// WithPassHostHeader sets the PassHostHeader field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the PassHostHeader field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithPassHostHeader(value bool) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.PassHostHeader = &value
	return b
}

// WithResponseForwarding sets the ResponseForwarding field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the ResponseForwarding field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithResponseForwarding(value *ResponseForwardingApplyConfiguration) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.ResponseForwarding = value
	return b
}

// WithServersTransport sets the ServersTransport field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the ServersTransport field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithServersTransport(value string) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.ServersTransport = &value
	return b
}

// WithWeight sets the Weight field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Weight field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithWeight(value int) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.Weight = &value
	return b
}

// WithNativeLB sets the NativeLB field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the NativeLB field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithNativeLB(value bool) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.NativeLB = &value
	return b
}

// WithNodePortLB sets the NodePortLB field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the NodePortLB field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithNodePortLB(value bool) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.NodePortLB = &value
	return b
}

// WithHealthCheck sets the HealthCheck field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the HealthCheck field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithHealthCheck(value *ServerHealthCheckApplyConfiguration) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.HealthCheck = value
	return b
}

// WithPassiveHealthCheck sets the PassiveHealthCheck field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the PassiveHealthCheck field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithPassiveHealthCheck(value *PassiveServerHealthCheckApplyConfiguration) *MirroringApplyConfiguration {
	b.LoadBalancerSpecApplyConfiguration.PassiveHealthCheck = value
	return b
}

// WithMirrorBody sets the MirrorBody field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the MirrorBody field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithMirrorBody(value bool) *MirroringApplyConfiguration {
	b.MirrorBody = &value
	return b
}

// WithMaxBodySize sets the MaxBodySize field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the MaxBodySize field is set to the value of the last call.
func (b *MirroringApplyConfiguration) WithMaxBodySize(value int64) *MirroringApplyConfiguration {
	b.MaxBodySize = &value
	return b
}

// WithMirrors adds the given value to the Mirrors field in the declarative configuration
// and returns the receiver, so that objects can be build by chaining "With" function invocations.
// If called multiple times, values provided by each call will be appended to the Mirrors field.
func (b *MirroringApplyConfiguration) WithMirrors(values ...*MirrorServiceApplyConfiguration) *MirroringApplyConfiguration {
	for i := range values {
		if values[i] == nil {
			panic("nil value passed to WithMirrors")
		}
		b.Mirrors = append(b.Mirrors, *values[i])
	}
	return b
}
