// Code generated by protoc-gen-goext. DO NOT EDIT.

package iam

import (
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
)

func (m *Service) SetServiceId(v string) {
	m.ServiceId = v
}

func (m *Service) SetResource(v *Resource) {
	m.Resource = v
}

func (m *Service) SetUpdatedAt(v *timestamppb.Timestamp) {
	m.UpdatedAt = v
}

func (m *Service) SetStatus(v Service_Status) {
	m.Status = v
}

func (m *ServiceAgent) SetServiceAccountId(v string) {
	m.ServiceAccountId = v
}

func (m *ServiceAgent) SetServiceId(v string) {
	m.ServiceId = v
}

func (m *ServiceAgent) SetMicroserviceId(v string) {
	m.MicroserviceId = v
}
