// Code Generated by gadget/xsdk, DO NOT EDIT

package dns

import (
	"context"
	"net/http"
)

type Client struct {
	caller Caller
}

type Caller interface {
	Do(*http.Request) (*http.Response, error)
}

func NewClient(c Caller) *Client {
	return &Client{caller: c}
}

func (c *Client) do(ctx context.Context, req *http.Request) (*http.Response, error) {
	return c.caller.Do(req)
}
