/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

import (
	apisv1 "sigs.k8s.io/gateway-api/apis/v1"
)

// ReferenceGrantFromApplyConfiguration represents a declarative configuration of the ReferenceGrantFrom type for use
// with apply.
//
// ReferenceGrantFrom describes trusted namespaces and kinds.
type ReferenceGrantFromApplyConfiguration struct {
	// Group is the group of the referent.
	// When empty, the Kubernetes core API group is inferred.
	//
	// Support: Core
	Group *apisv1.Group `json:"group,omitempty"`
	// Kind is the kind of the referent. Although implementations may support
	// additional resources, the following types are part of the "Core"
	// support level for this field.
	//
	// When used to permit a SecretObjectReference:
	//
	// * Gateway
	//
	// When used to permit a BackendObjectReference:
	//
	// * GRPCRoute
	// * HTTPRoute
	// * TCPRoute
	// * TLSRoute
	// * UDPRoute
	Kind *apisv1.Kind `json:"kind,omitempty"`
	// Namespace is the namespace of the referent.
	//
	// Support: Core
	Namespace *apisv1.Namespace `json:"namespace,omitempty"`
}

// ReferenceGrantFromApplyConfiguration constructs a declarative configuration of the ReferenceGrantFrom type for use with
// apply.
func ReferenceGrantFrom() *ReferenceGrantFromApplyConfiguration {
	return &ReferenceGrantFromApplyConfiguration{}
}

// WithGroup sets the Group field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Group field is set to the value of the last call.
func (b *ReferenceGrantFromApplyConfiguration) WithGroup(value apisv1.Group) *ReferenceGrantFromApplyConfiguration {
	b.Group = &value
	return b
}

// WithKind sets the Kind field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Kind field is set to the value of the last call.
func (b *ReferenceGrantFromApplyConfiguration) WithKind(value apisv1.Kind) *ReferenceGrantFromApplyConfiguration {
	b.Kind = &value
	return b
}

// WithNamespace sets the Namespace field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Namespace field is set to the value of the last call.
func (b *ReferenceGrantFromApplyConfiguration) WithNamespace(value apisv1.Namespace) *ReferenceGrantFromApplyConfiguration {
	b.Namespace = &value
	return b
}
