// Copyright 2018-2025 JDCLOUD.COM
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package core

import "time"

type Config struct {
	Scheme   string
	Endpoint string
	Timeout  time.Duration
}

// NewConfig returns a pointer of Config
//
// scheme only accepts http or https
//
// endpoint is the host to access, the connection could not be created if it's error
func NewConfig() *Config {
	return &Config{SchemeHttps, "www.jdcloud-api.com", 10 * time.Second}
}

func (c *Config) SetScheme(scheme string) {
	c.Scheme = scheme
}

func (c *Config) SetEndpoint(endpoint string) {
	c.Endpoint = endpoint
}

func (c *Config) SetTimeout(timeout time.Duration) {
	c.Timeout = timeout
}
