/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

import (
	apisv1 "sigs.k8s.io/gateway-api/apis/v1"
)

// GatewayClassSpecApplyConfiguration represents a declarative configuration of the GatewayClassSpec type for use
// with apply.
//
// GatewayClassSpec reflects the configuration of a class of Gateways.
type GatewayClassSpecApplyConfiguration struct {
	// ControllerName is the name of the controller that is managing Gateways of
	// this class. The value of this field MUST be a domain prefixed path.
	//
	// Example: "example.net/gateway-controller".
	//
	// This field is not mutable and cannot be empty.
	//
	// Support: Core
	ControllerName *apisv1.GatewayController `json:"controllerName,omitempty"`
	// ParametersRef is a reference to a resource that contains the configuration
	// parameters corresponding to the GatewayClass. This is optional if the
	// controller does not require any additional configuration.
	//
	// ParametersRef can reference a standard Kubernetes resource, i.e. ConfigMap,
	// or an implementation-specific custom resource. The resource can be
	// cluster-scoped or namespace-scoped.
	//
	// If the referent cannot be found, refers to an unsupported kind, or when
	// the data within that resource is malformed, the GatewayClass SHOULD be
	// rejected with the "Accepted" status condition set to "False" and an
	// "InvalidParameters" reason.
	//
	// A Gateway for this GatewayClass may provide its own `parametersRef`. When both are specified,
	// the merging behavior is implementation specific.
	// It is generally recommended that GatewayClass provides defaults that can be overridden by a Gateway.
	//
	// Support: Implementation-specific
	ParametersRef *ParametersReferenceApplyConfiguration `json:"parametersRef,omitempty"`
	// Description helps describe a GatewayClass with more details.
	Description *string `json:"description,omitempty"`
}

// GatewayClassSpecApplyConfiguration constructs a declarative configuration of the GatewayClassSpec type for use with
// apply.
func GatewayClassSpec() *GatewayClassSpecApplyConfiguration {
	return &GatewayClassSpecApplyConfiguration{}
}

// WithControllerName sets the ControllerName field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the ControllerName field is set to the value of the last call.
func (b *GatewayClassSpecApplyConfiguration) WithControllerName(value apisv1.GatewayController) *GatewayClassSpecApplyConfiguration {
	b.ControllerName = &value
	return b
}

// WithParametersRef sets the ParametersRef field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the ParametersRef field is set to the value of the last call.
func (b *GatewayClassSpecApplyConfiguration) WithParametersRef(value *ParametersReferenceApplyConfiguration) *GatewayClassSpecApplyConfiguration {
	b.ParametersRef = value
	return b
}

// WithDescription sets the Description field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Description field is set to the value of the last call.
func (b *GatewayClassSpecApplyConfiguration) WithDescription(value string) *GatewayClassSpecApplyConfiguration {
	b.Description = &value
	return b
}
