// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v3.21.12
// source: yandex/cloud/iam/v1/oauth_client_service.proto

package iam

import (
	_ "github.com/yandex-cloud/go-genproto/yandex/cloud"
	_ "github.com/yandex-cloud/go-genproto/yandex/cloud/api"
	operation "github.com/yandex-cloud/go-genproto/yandex/cloud/operation"
	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	fieldmaskpb "google.golang.org/protobuf/types/known/fieldmaskpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type GetOAuthClientRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the OAuthClient resource to return.
	// To get the oauth client ID, use a [OAuthClientService.List] request.
	OauthClientId string `protobuf:"bytes,1,opt,name=oauth_client_id,json=oauthClientId,proto3" json:"oauth_client_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GetOAuthClientRequest) Reset() {
	*x = GetOAuthClientRequest{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GetOAuthClientRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GetOAuthClientRequest) ProtoMessage() {}

func (x *GetOAuthClientRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GetOAuthClientRequest.ProtoReflect.Descriptor instead.
func (*GetOAuthClientRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescGZIP(), []int{0}
}

func (x *GetOAuthClientRequest) GetOauthClientId() string {
	if x != nil {
		return x.OauthClientId
	}
	return ""
}

type ListOAuthClientsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The maximum number of results per page to return. If the number of available
	// results is larger than [page_size],
	// the service returns a [ListOAuthClientsResponse.next_page_token]
	// that can be used to get the next page of results in subsequent list requests.
	PageSize int64 `protobuf:"varint,1,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// Page token. To get the next page of results, set [page_token] to the
	// [ListOAuthClientsResponse.next_page_token] returned by a previous list request.
	PageToken string `protobuf:"bytes,2,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	// ID of the folder to list oauth clients for
	// To get the folder ID, use a [yandex.cloud.resourcemanager.v1.FolderService.List] request.
	FolderId      string `protobuf:"bytes,3,opt,name=folder_id,json=folderId,proto3" json:"folder_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListOAuthClientsRequest) Reset() {
	*x = ListOAuthClientsRequest{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListOAuthClientsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListOAuthClientsRequest) ProtoMessage() {}

func (x *ListOAuthClientsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListOAuthClientsRequest.ProtoReflect.Descriptor instead.
func (*ListOAuthClientsRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescGZIP(), []int{1}
}

func (x *ListOAuthClientsRequest) GetPageSize() int64 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListOAuthClientsRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

func (x *ListOAuthClientsRequest) GetFolderId() string {
	if x != nil {
		return x.FolderId
	}
	return ""
}

type ListOAuthClientsResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// List of OAuthClient resource views
	OauthClients []*OAuthClientListView `protobuf:"bytes,1,rep,name=oauth_clients,json=oauthClients,proto3" json:"oauth_clients,omitempty"`
	// This token allows you to get the next page of results for list requests. If the number of results
	// is larger than [ListOAuthClientsRequest.page_size], use
	// the [next_page_token] as the value
	// for the [ListOAuthClientsRequest.page_token] query parameter
	// in the next list request. Each subsequent list request will have its own
	// [next_page_token] to continue paging through the results.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListOAuthClientsResponse) Reset() {
	*x = ListOAuthClientsResponse{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListOAuthClientsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListOAuthClientsResponse) ProtoMessage() {}

func (x *ListOAuthClientsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListOAuthClientsResponse.ProtoReflect.Descriptor instead.
func (*ListOAuthClientsResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescGZIP(), []int{2}
}

func (x *ListOAuthClientsResponse) GetOauthClients() []*OAuthClientListView {
	if x != nil {
		return x.OauthClients
	}
	return nil
}

func (x *ListOAuthClientsResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

type OAuthClientListView struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the OAuthClient resource.
	Id string `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	// Name of the OAuthClient resource.
	Name          string `protobuf:"bytes,2,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *OAuthClientListView) Reset() {
	*x = OAuthClientListView{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OAuthClientListView) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OAuthClientListView) ProtoMessage() {}

func (x *OAuthClientListView) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OAuthClientListView.ProtoReflect.Descriptor instead.
func (*OAuthClientListView) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescGZIP(), []int{3}
}

func (x *OAuthClientListView) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

func (x *OAuthClientListView) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

type UpdateOAuthClientMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the oauth client that is being updated
	OauthClientId string `protobuf:"bytes,1,opt,name=oauth_client_id,json=oauthClientId,proto3" json:"oauth_client_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateOAuthClientMetadata) Reset() {
	*x = UpdateOAuthClientMetadata{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateOAuthClientMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateOAuthClientMetadata) ProtoMessage() {}

func (x *UpdateOAuthClientMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateOAuthClientMetadata.ProtoReflect.Descriptor instead.
func (*UpdateOAuthClientMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescGZIP(), []int{4}
}

func (x *UpdateOAuthClientMetadata) GetOauthClientId() string {
	if x != nil {
		return x.OauthClientId
	}
	return ""
}

type DeleteOAuthClientMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the oauth client that is being deleted
	OauthClientId string `protobuf:"bytes,1,opt,name=oauth_client_id,json=oauthClientId,proto3" json:"oauth_client_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DeleteOAuthClientMetadata) Reset() {
	*x = DeleteOAuthClientMetadata{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DeleteOAuthClientMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DeleteOAuthClientMetadata) ProtoMessage() {}

func (x *DeleteOAuthClientMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DeleteOAuthClientMetadata.ProtoReflect.Descriptor instead.
func (*DeleteOAuthClientMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescGZIP(), []int{5}
}

func (x *DeleteOAuthClientMetadata) GetOauthClientId() string {
	if x != nil {
		return x.OauthClientId
	}
	return ""
}

type CreateOAuthClientRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Name of the oauth client.
	// The name must be unique within folder.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// List of redirect uries allowed for the oauth client.
	RedirectUris []string `protobuf:"bytes,2,rep,name=redirect_uris,json=redirectUris,proto3" json:"redirect_uris,omitempty"`
	// List of oauth scopes requested by the oauth client.
	Scopes []string `protobuf:"bytes,3,rep,name=scopes,proto3" json:"scopes,omitempty"`
	// ID of the folder to create an oauth client in.
	// To get the folder ID, use a [yandex.cloud.resourcemanager.v1.FolderService.List] request.
	FolderId      string `protobuf:"bytes,4,opt,name=folder_id,json=folderId,proto3" json:"folder_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateOAuthClientRequest) Reset() {
	*x = CreateOAuthClientRequest{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateOAuthClientRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateOAuthClientRequest) ProtoMessage() {}

func (x *CreateOAuthClientRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateOAuthClientRequest.ProtoReflect.Descriptor instead.
func (*CreateOAuthClientRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescGZIP(), []int{6}
}

func (x *CreateOAuthClientRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *CreateOAuthClientRequest) GetRedirectUris() []string {
	if x != nil {
		return x.RedirectUris
	}
	return nil
}

func (x *CreateOAuthClientRequest) GetScopes() []string {
	if x != nil {
		return x.Scopes
	}
	return nil
}

func (x *CreateOAuthClientRequest) GetFolderId() string {
	if x != nil {
		return x.FolderId
	}
	return ""
}

type CreateOAuthClientMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the oauth client that is being created
	OauthClientId string `protobuf:"bytes,1,opt,name=oauth_client_id,json=oauthClientId,proto3" json:"oauth_client_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CreateOAuthClientMetadata) Reset() {
	*x = CreateOAuthClientMetadata{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CreateOAuthClientMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CreateOAuthClientMetadata) ProtoMessage() {}

func (x *CreateOAuthClientMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CreateOAuthClientMetadata.ProtoReflect.Descriptor instead.
func (*CreateOAuthClientMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescGZIP(), []int{7}
}

func (x *CreateOAuthClientMetadata) GetOauthClientId() string {
	if x != nil {
		return x.OauthClientId
	}
	return ""
}

type UpdateOAuthClientRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the OAuthClient resource to update.
	// To get the oauth client ID, use a [OAuthClientService.List] request.
	OauthClientId string `protobuf:"bytes,1,opt,name=oauth_client_id,json=oauthClientId,proto3" json:"oauth_client_id,omitempty"`
	// Field mask that specifies which fields of the OAuthClient resource are going to be updated.
	UpdateMask *fieldmaskpb.FieldMask `protobuf:"bytes,2,opt,name=update_mask,json=updateMask,proto3" json:"update_mask,omitempty"`
	// Name of the oauth client.
	// The name must be unique within folder.
	Name string `protobuf:"bytes,3,opt,name=name,proto3" json:"name,omitempty"`
	// List of redirect uries allowed for the oauth client.
	RedirectUris []string `protobuf:"bytes,4,rep,name=redirect_uris,json=redirectUris,proto3" json:"redirect_uris,omitempty"`
	// List of oauth scopes requested by the oauth client.
	Scopes        []string `protobuf:"bytes,5,rep,name=scopes,proto3" json:"scopes,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateOAuthClientRequest) Reset() {
	*x = UpdateOAuthClientRequest{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateOAuthClientRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateOAuthClientRequest) ProtoMessage() {}

func (x *UpdateOAuthClientRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateOAuthClientRequest.ProtoReflect.Descriptor instead.
func (*UpdateOAuthClientRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescGZIP(), []int{8}
}

func (x *UpdateOAuthClientRequest) GetOauthClientId() string {
	if x != nil {
		return x.OauthClientId
	}
	return ""
}

func (x *UpdateOAuthClientRequest) GetUpdateMask() *fieldmaskpb.FieldMask {
	if x != nil {
		return x.UpdateMask
	}
	return nil
}

func (x *UpdateOAuthClientRequest) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *UpdateOAuthClientRequest) GetRedirectUris() []string {
	if x != nil {
		return x.RedirectUris
	}
	return nil
}

func (x *UpdateOAuthClientRequest) GetScopes() []string {
	if x != nil {
		return x.Scopes
	}
	return nil
}

type DeleteOAuthClientRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the oauth client to delete.
	// To get the oauth client ID, use [OAuthClientService.list] request.
	OauthClientId string `protobuf:"bytes,1,opt,name=oauth_client_id,json=oauthClientId,proto3" json:"oauth_client_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DeleteOAuthClientRequest) Reset() {
	*x = DeleteOAuthClientRequest{}
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DeleteOAuthClientRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DeleteOAuthClientRequest) ProtoMessage() {}

func (x *DeleteOAuthClientRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DeleteOAuthClientRequest.ProtoReflect.Descriptor instead.
func (*DeleteOAuthClientRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescGZIP(), []int{9}
}

func (x *DeleteOAuthClientRequest) GetOauthClientId() string {
	if x != nil {
		return x.OauthClientId
	}
	return ""
}

var File_yandex_cloud_iam_v1_oauth_client_service_proto protoreflect.FileDescriptor

const file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDesc = "" +
	"\n" +
	".yandex/cloud/iam/v1/oauth_client_service.proto\x12\x13yandex.cloud.iam.v1\x1a&yandex/cloud/iam/v1/oauth_client.proto\x1a\x1cgoogle/api/annotations.proto\x1a google/protobuf/field_mask.proto\x1a yandex/cloud/api/operation.proto\x1a&yandex/cloud/operation/operation.proto\x1a\x1dyandex/cloud/validation.proto\"M\n" +
	"\x15GetOAuthClientRequest\x124\n" +
	"\x0foauth_client_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\roauthClientId\"\x98\x01\n" +
	"\x17ListOAuthClientsRequest\x12'\n" +
	"\tpage_size\x18\x01 \x01(\x03B\n" +
	"\xfa\xc71\x060-1000R\bpageSize\x12)\n" +
	"\n" +
	"page_token\x18\x02 \x01(\tB\n" +
	"\x8a\xc81\x06<=2000R\tpageToken\x12)\n" +
	"\tfolder_id\x18\x03 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\bfolderId\"\x91\x01\n" +
	"\x18ListOAuthClientsResponse\x12M\n" +
	"\roauth_clients\x18\x01 \x03(\v2(.yandex.cloud.iam.v1.OAuthClientListViewR\foauthClients\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"9\n" +
	"\x13OAuthClientListView\x12\x0e\n" +
	"\x02id\x18\x01 \x01(\tR\x02id\x12\x12\n" +
	"\x04name\x18\x02 \x01(\tR\x04name\"C\n" +
	"\x19UpdateOAuthClientMetadata\x12&\n" +
	"\x0foauth_client_id\x18\x01 \x01(\tR\roauthClientId\"C\n" +
	"\x19DeleteOAuthClientMetadata\x12&\n" +
	"\x0foauth_client_id\x18\x01 \x01(\tR\roauthClientId\"\xeb\x01\n" +
	"\x18CreateOAuthClientRequest\x12;\n" +
	"\x04name\x18\x01 \x01(\tB'\xe8\xc71\x01\xf2\xc71\x1f[a-z]([-a-z0-9]{0,61}[a-z0-9])?R\x04name\x129\n" +
	"\rredirect_uris\x18\x02 \x03(\tB\x14\x82\xc81\x06<=1000\x8a\xc81\x06<=1000R\fredirectUris\x12+\n" +
	"\x06scopes\x18\x03 \x03(\tB\x13\x82\xc81\x06<=1000\x8a\xc81\x05<=255R\x06scopes\x12*\n" +
	"\tfolder_id\x18\x04 \x01(\tB\r\xe8\xc71\x01\x8a\xc81\x05<=255R\bfolderId\"C\n" +
	"\x19CreateOAuthClientMetadata\x12&\n" +
	"\x0foauth_client_id\x18\x01 \x01(\tR\roauthClientId\"\xaa\x02\n" +
	"\x18UpdateOAuthClientRequest\x12,\n" +
	"\x0foauth_client_id\x18\x01 \x01(\tB\x04\xe8\xc71\x01R\roauthClientId\x12;\n" +
	"\vupdate_mask\x18\x02 \x01(\v2\x1a.google.protobuf.FieldMaskR\n" +
	"updateMask\x12;\n" +
	"\x04name\x18\x03 \x01(\tB'\xe8\xc71\x01\xf2\xc71\x1f[a-z]([-a-z0-9]{0,61}[a-z0-9])?R\x04name\x129\n" +
	"\rredirect_uris\x18\x04 \x03(\tB\x14\x82\xc81\x06<=1000\x8a\xc81\x06<=1000R\fredirectUris\x12+\n" +
	"\x06scopes\x18\x05 \x03(\tB\x13\x82\xc81\x06<=1000\x8a\xc81\x05<=255R\x06scopes\"P\n" +
	"\x18DeleteOAuthClientRequest\x124\n" +
	"\x0foauth_client_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\roauthClientId2\xc7\x06\n" +
	"\x12OAuthClientService\x12\x83\x01\n" +
	"\x03Get\x12*.yandex.cloud.iam.v1.GetOAuthClientRequest\x1a .yandex.cloud.iam.v1.OAuthClient\".\x82\xd3\xe4\x93\x02(\x12&/iam/v1/oauthClients/{oauth_client_id}\x12\x81\x01\n" +
	"\x04List\x12,.yandex.cloud.iam.v1.ListOAuthClientsRequest\x1a-.yandex.cloud.iam.v1.ListOAuthClientsResponse\"\x1c\x82\xd3\xe4\x93\x02\x16\x12\x14/iam/v1/oauthClients\x12\xa7\x01\n" +
	"\x06Create\x12-.yandex.cloud.iam.v1.CreateOAuthClientRequest\x1a!.yandex.cloud.operation.Operation\"K\xb2\xd2*(\n" +
	"\x19CreateOAuthClientMetadata\x12\vOAuthClient\x82\xd3\xe4\x93\x02\x19:\x01*\"\x14/iam/v1/oauthClients\x12\xb9\x01\n" +
	"\x06Update\x12-.yandex.cloud.iam.v1.UpdateOAuthClientRequest\x1a!.yandex.cloud.operation.Operation\"]\xb2\xd2*(\n" +
	"\x19UpdateOAuthClientMetadata\x12\vOAuthClient\x82\xd3\xe4\x93\x02+:\x01*2&/iam/v1/oauthClients/{oauth_client_id}\x12\xc0\x01\n" +
	"\x06Delete\x12-.yandex.cloud.iam.v1.DeleteOAuthClientRequest\x1a!.yandex.cloud.operation.Operation\"d\xb2\xd2*2\n" +
	"\x19DeleteOAuthClientMetadata\x12\x15google.protobuf.Empty\x82\xd3\xe4\x93\x02(*&/iam/v1/oauthClients/{oauth_client_id}BV\n" +
	"\x17yandex.cloud.api.iam.v1Z;github.com/yandex-cloud/go-genproto/yandex/cloud/iam/v1;iamb\x06proto3"

var (
	file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescOnce sync.Once
	file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescData []byte
)

func file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescGZIP() []byte {
	file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescOnce.Do(func() {
		file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDesc), len(file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDesc)))
	})
	return file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDescData
}

var file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes = make([]protoimpl.MessageInfo, 10)
var file_yandex_cloud_iam_v1_oauth_client_service_proto_goTypes = []any{
	(*GetOAuthClientRequest)(nil),     // 0: yandex.cloud.iam.v1.GetOAuthClientRequest
	(*ListOAuthClientsRequest)(nil),   // 1: yandex.cloud.iam.v1.ListOAuthClientsRequest
	(*ListOAuthClientsResponse)(nil),  // 2: yandex.cloud.iam.v1.ListOAuthClientsResponse
	(*OAuthClientListView)(nil),       // 3: yandex.cloud.iam.v1.OAuthClientListView
	(*UpdateOAuthClientMetadata)(nil), // 4: yandex.cloud.iam.v1.UpdateOAuthClientMetadata
	(*DeleteOAuthClientMetadata)(nil), // 5: yandex.cloud.iam.v1.DeleteOAuthClientMetadata
	(*CreateOAuthClientRequest)(nil),  // 6: yandex.cloud.iam.v1.CreateOAuthClientRequest
	(*CreateOAuthClientMetadata)(nil), // 7: yandex.cloud.iam.v1.CreateOAuthClientMetadata
	(*UpdateOAuthClientRequest)(nil),  // 8: yandex.cloud.iam.v1.UpdateOAuthClientRequest
	(*DeleteOAuthClientRequest)(nil),  // 9: yandex.cloud.iam.v1.DeleteOAuthClientRequest
	(*fieldmaskpb.FieldMask)(nil),     // 10: google.protobuf.FieldMask
	(*OAuthClient)(nil),               // 11: yandex.cloud.iam.v1.OAuthClient
	(*operation.Operation)(nil),       // 12: yandex.cloud.operation.Operation
}
var file_yandex_cloud_iam_v1_oauth_client_service_proto_depIdxs = []int32{
	3,  // 0: yandex.cloud.iam.v1.ListOAuthClientsResponse.oauth_clients:type_name -> yandex.cloud.iam.v1.OAuthClientListView
	10, // 1: yandex.cloud.iam.v1.UpdateOAuthClientRequest.update_mask:type_name -> google.protobuf.FieldMask
	0,  // 2: yandex.cloud.iam.v1.OAuthClientService.Get:input_type -> yandex.cloud.iam.v1.GetOAuthClientRequest
	1,  // 3: yandex.cloud.iam.v1.OAuthClientService.List:input_type -> yandex.cloud.iam.v1.ListOAuthClientsRequest
	6,  // 4: yandex.cloud.iam.v1.OAuthClientService.Create:input_type -> yandex.cloud.iam.v1.CreateOAuthClientRequest
	8,  // 5: yandex.cloud.iam.v1.OAuthClientService.Update:input_type -> yandex.cloud.iam.v1.UpdateOAuthClientRequest
	9,  // 6: yandex.cloud.iam.v1.OAuthClientService.Delete:input_type -> yandex.cloud.iam.v1.DeleteOAuthClientRequest
	11, // 7: yandex.cloud.iam.v1.OAuthClientService.Get:output_type -> yandex.cloud.iam.v1.OAuthClient
	2,  // 8: yandex.cloud.iam.v1.OAuthClientService.List:output_type -> yandex.cloud.iam.v1.ListOAuthClientsResponse
	12, // 9: yandex.cloud.iam.v1.OAuthClientService.Create:output_type -> yandex.cloud.operation.Operation
	12, // 10: yandex.cloud.iam.v1.OAuthClientService.Update:output_type -> yandex.cloud.operation.Operation
	12, // 11: yandex.cloud.iam.v1.OAuthClientService.Delete:output_type -> yandex.cloud.operation.Operation
	7,  // [7:12] is the sub-list for method output_type
	2,  // [2:7] is the sub-list for method input_type
	2,  // [2:2] is the sub-list for extension type_name
	2,  // [2:2] is the sub-list for extension extendee
	0,  // [0:2] is the sub-list for field type_name
}

func init() { file_yandex_cloud_iam_v1_oauth_client_service_proto_init() }
func file_yandex_cloud_iam_v1_oauth_client_service_proto_init() {
	if File_yandex_cloud_iam_v1_oauth_client_service_proto != nil {
		return
	}
	file_yandex_cloud_iam_v1_oauth_client_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDesc), len(file_yandex_cloud_iam_v1_oauth_client_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   10,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_yandex_cloud_iam_v1_oauth_client_service_proto_goTypes,
		DependencyIndexes: file_yandex_cloud_iam_v1_oauth_client_service_proto_depIdxs,
		MessageInfos:      file_yandex_cloud_iam_v1_oauth_client_service_proto_msgTypes,
	}.Build()
	File_yandex_cloud_iam_v1_oauth_client_service_proto = out.File
	file_yandex_cloud_iam_v1_oauth_client_service_proto_goTypes = nil
	file_yandex_cloud_iam_v1_oauth_client_service_proto_depIdxs = nil
}
