// Code generated by protoc-gen-goext. DO NOT EDIT.

package iam

func (m *RevokeRefreshTokenMetadata) SetSubjectId(v string) {
	m.SubjectId = v
}

func (m *RevokeRefreshTokenMetadata) SetRefreshTokenIds(v []string) {
	m.RefreshTokenIds = v
}

func (m *ListRefreshTokensRequest) SetSubjectId(v string) {
	m.SubjectId = v
}

func (m *ListRefreshTokensRequest) SetPageSize(v int64) {
	m.PageSize = v
}

func (m *ListRefreshTokensRequest) SetPageToken(v string) {
	m.PageToken = v
}

func (m *ListRefreshTokensRequest) SetFilter(v string) {
	m.Filter = v
}

func (m *ListRefreshTokensResponse) SetRefreshTokens(v []*RefreshToken) {
	m.RefreshTokens = v
}

func (m *ListRefreshTokensResponse) SetNextPageToken(v string) {
	m.NextPageToken = v
}

type RevokeRefreshTokenRequest_Filter = isRevokeRefreshTokenRequest_Filter

func (m *RevokeRefreshTokenRequest) SetFilter(v RevokeRefreshTokenRequest_Filter) {
	m.Filter = v
}

func (m *RevokeRefreshTokenRequest) SetRefreshTokenId(v string) {
	m.Filter = &RevokeRefreshTokenRequest_RefreshTokenId{
		RefreshTokenId: v,
	}
}

func (m *RevokeRefreshTokenRequest) SetRefreshToken(v string) {
	m.Filter = &RevokeRefreshTokenRequest_RefreshToken{
		RefreshToken: v,
	}
}

func (m *RevokeRefreshTokenRequest) SetRevokeFilter(v *RevokeFilter) {
	m.Filter = &RevokeRefreshTokenRequest_RevokeFilter{
		RevokeFilter: v,
	}
}

func (m *RevokeFilter) SetClientId(v string) {
	m.ClientId = v
}

func (m *RevokeFilter) SetSubjectId(v string) {
	m.SubjectId = v
}

func (m *RevokeFilter) SetClientInstanceInfo(v string) {
	m.ClientInstanceInfo = v
}

func (m *RevokeRefreshTokenResponse) SetRefreshTokenIds(v []string) {
	m.RefreshTokenIds = v
}
