// Copyright 2020-2021 InfluxData, Inc. All rights reserved.
// Use of this source code is governed by MIT
// license that can be found in the LICENSE file.

// Package http hold internal HTTP related stuff
package http

// UserAgent keeps once created User-Agent string
var UserAgent string
