// Code generated by protoc-gen-goext. DO NOT EDIT.

package cloud

func (m *MapKeySpec) SetValue(v string) {
	m.Value = v
}

func (m *MapKeySpec) SetPattern(v string) {
	m.Pattern = v
}

func (m *MapKeySpec) SetLength(v string) {
	m.Length = v
}
