// Code generated by protoc-gen-goext. DO NOT EDIT.

package iam

func (m *GetOAuthClientSecretRequest) SetOauthClientSecretId(v string) {
	m.OauthClientSecretId = v
}

func (m *ListOAuthClientSecretsRequest) SetPageSize(v int64) {
	m.PageSize = v
}

func (m *ListOAuthClientSecretsRequest) SetPageToken(v string) {
	m.PageToken = v
}

func (m *ListOAuthClientSecretsRequest) SetOauthClientId(v string) {
	m.OauthClientId = v
}

func (m *ListOAuthClientSecretsResponse) SetOauthClientSecrets(v []*OAuthClientSecret) {
	m.OauthClientSecrets = v
}

func (m *ListOAuthClientSecretsResponse) SetNextPageToken(v string) {
	m.NextPageToken = v
}

func (m *CreateOAuthClientSecretRequest) SetOauthClientId(v string) {
	m.OauthClientId = v
}

func (m *CreateOAuthClientSecretRequest) SetDescription(v string) {
	m.Description = v
}

func (m *CreateOAuthClientSecretMetadata) SetOauthClientSecretId(v string) {
	m.OauthClientSecretId = v
}

func (m *CreateOAuthClientSecretResponse) SetOauthClientSecret(v *OAuthClientSecret) {
	m.OauthClientSecret = v
}

func (m *CreateOAuthClientSecretResponse) SetSecretValue(v string) {
	m.SecretValue = v
}

func (m *DeleteOAuthClientSecretRequest) SetOauthClientSecretId(v string) {
	m.OauthClientSecretId = v
}

func (m *DeleteOAuthClientSecretMetadata) SetOauthClientSecretId(v string) {
	m.OauthClientSecretId = v
}
