/*
Intelligent DNS API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

API version: 1.0.0
*/

// Code generated by OpenAPI Generator (https://openapi-generator.tech); DO NOT EDIT.

package idns

import (
	"encoding/json"
)

// checks if the GetOrPatchDnsSecResponse type satisfies the MappedNullable interface at compile time
var _ MappedNullable = &GetOrPatchDnsSecResponse{}

// GetOrPatchDnsSecResponse Object returned by get zone DNSSEC
type GetOrPatchDnsSecResponse struct {
	// The schema version
	SchemaVersion *int32 `json:"schema_version,omitempty"`
	Results *DnsSec `json:"results,omitempty"`
}

// NewGetOrPatchDnsSecResponse instantiates a new GetOrPatchDnsSecResponse object
// This constructor will assign default values to properties that have it defined,
// and makes sure properties required by API are set, but the set of arguments
// will change when the set of required properties is changed
func NewGetOrPatchDnsSecResponse() *GetOrPatchDnsSecResponse {
	this := GetOrPatchDnsSecResponse{}
	return &this
}

// NewGetOrPatchDnsSecResponseWithDefaults instantiates a new GetOrPatchDnsSecResponse object
// This constructor will only assign default values to properties that have it defined,
// but it doesn't guarantee that properties required by API are set
func NewGetOrPatchDnsSecResponseWithDefaults() *GetOrPatchDnsSecResponse {
	this := GetOrPatchDnsSecResponse{}
	return &this
}

// GetSchemaVersion returns the SchemaVersion field value if set, zero value otherwise.
func (o *GetOrPatchDnsSecResponse) GetSchemaVersion() int32 {
	if o == nil || IsNil(o.SchemaVersion) {
		var ret int32
		return ret
	}
	return *o.SchemaVersion
}

// GetSchemaVersionOk returns a tuple with the SchemaVersion field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *GetOrPatchDnsSecResponse) GetSchemaVersionOk() (*int32, bool) {
	if o == nil || IsNil(o.SchemaVersion) {
		return nil, false
	}
	return o.SchemaVersion, true
}

// HasSchemaVersion returns a boolean if a field has been set.
func (o *GetOrPatchDnsSecResponse) HasSchemaVersion() bool {
	if o != nil && !IsNil(o.SchemaVersion) {
		return true
	}

	return false
}

// SetSchemaVersion gets a reference to the given int32 and assigns it to the SchemaVersion field.
func (o *GetOrPatchDnsSecResponse) SetSchemaVersion(v int32) {
	o.SchemaVersion = &v
}

// GetResults returns the Results field value if set, zero value otherwise.
func (o *GetOrPatchDnsSecResponse) GetResults() DnsSec {
	if o == nil || IsNil(o.Results) {
		var ret DnsSec
		return ret
	}
	return *o.Results
}

// GetResultsOk returns a tuple with the Results field value if set, nil otherwise
// and a boolean to check if the value has been set.
func (o *GetOrPatchDnsSecResponse) GetResultsOk() (*DnsSec, bool) {
	if o == nil || IsNil(o.Results) {
		return nil, false
	}
	return o.Results, true
}

// HasResults returns a boolean if a field has been set.
func (o *GetOrPatchDnsSecResponse) HasResults() bool {
	if o != nil && !IsNil(o.Results) {
		return true
	}

	return false
}

// SetResults gets a reference to the given DnsSec and assigns it to the Results field.
func (o *GetOrPatchDnsSecResponse) SetResults(v DnsSec) {
	o.Results = &v
}

func (o GetOrPatchDnsSecResponse) MarshalJSON() ([]byte, error) {
	toSerialize,err := o.ToMap()
	if err != nil {
		return []byte{}, err
	}
	return json.Marshal(toSerialize)
}

func (o GetOrPatchDnsSecResponse) ToMap() (map[string]interface{}, error) {
	toSerialize := map[string]interface{}{}
	if !IsNil(o.SchemaVersion) {
		toSerialize["schema_version"] = o.SchemaVersion
	}
	if !IsNil(o.Results) {
		toSerialize["results"] = o.Results
	}
	return toSerialize, nil
}

type NullableGetOrPatchDnsSecResponse struct {
	value *GetOrPatchDnsSecResponse
	isSet bool
}

func (v NullableGetOrPatchDnsSecResponse) Get() *GetOrPatchDnsSecResponse {
	return v.value
}

func (v *NullableGetOrPatchDnsSecResponse) Set(val *GetOrPatchDnsSecResponse) {
	v.value = val
	v.isSet = true
}

func (v NullableGetOrPatchDnsSecResponse) IsSet() bool {
	return v.isSet
}

func (v *NullableGetOrPatchDnsSecResponse) Unset() {
	v.value = nil
	v.isSet = false
}

func NewNullableGetOrPatchDnsSecResponse(val *GetOrPatchDnsSecResponse) *NullableGetOrPatchDnsSecResponse {
	return &NullableGetOrPatchDnsSecResponse{value: val, isSet: true}
}

func (v NullableGetOrPatchDnsSecResponse) MarshalJSON() ([]byte, error) {
	return json.Marshal(v.value)
}

func (v *NullableGetOrPatchDnsSecResponse) UnmarshalJSON(src []byte) error {
	v.isSet = true
	return json.Unmarshal(src, &v.value)
}


