// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v3.21.12
// source: yandex/cloud/dns/v1/dns_firewall.proto

package dns

import (
	_ "github.com/yandex-cloud/go-genproto/yandex/cloud"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type DnsFirewall_ResourceConfig_ResourceType int32

const (
	// Unspecified resource type.
	DnsFirewall_ResourceConfig_RESOURCE_TYPE_UNSPECIFIED DnsFirewall_ResourceConfig_ResourceType = 0
	// VPC network resource type.
	DnsFirewall_ResourceConfig_NETWORK DnsFirewall_ResourceConfig_ResourceType = 1
	// Folder resource type.
	DnsFirewall_ResourceConfig_FOLDER DnsFirewall_ResourceConfig_ResourceType = 2
	// Cloud resource type.
	DnsFirewall_ResourceConfig_CLOUD DnsFirewall_ResourceConfig_ResourceType = 3
)

// Enum value maps for DnsFirewall_ResourceConfig_ResourceType.
var (
	DnsFirewall_ResourceConfig_ResourceType_name = map[int32]string{
		0: "RESOURCE_TYPE_UNSPECIFIED",
		1: "NETWORK",
		2: "FOLDER",
		3: "CLOUD",
	}
	DnsFirewall_ResourceConfig_ResourceType_value = map[string]int32{
		"RESOURCE_TYPE_UNSPECIFIED": 0,
		"NETWORK":                   1,
		"FOLDER":                    2,
		"CLOUD":                     3,
	}
)

func (x DnsFirewall_ResourceConfig_ResourceType) Enum() *DnsFirewall_ResourceConfig_ResourceType {
	p := new(DnsFirewall_ResourceConfig_ResourceType)
	*p = x
	return p
}

func (x DnsFirewall_ResourceConfig_ResourceType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (DnsFirewall_ResourceConfig_ResourceType) Descriptor() protoreflect.EnumDescriptor {
	return file_yandex_cloud_dns_v1_dns_firewall_proto_enumTypes[0].Descriptor()
}

func (DnsFirewall_ResourceConfig_ResourceType) Type() protoreflect.EnumType {
	return &file_yandex_cloud_dns_v1_dns_firewall_proto_enumTypes[0]
}

func (x DnsFirewall_ResourceConfig_ResourceType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use DnsFirewall_ResourceConfig_ResourceType.Descriptor instead.
func (DnsFirewall_ResourceConfig_ResourceType) EnumDescriptor() ([]byte, []int) {
	return file_yandex_cloud_dns_v1_dns_firewall_proto_rawDescGZIP(), []int{0, 0, 0}
}

// A DNS firewall. For details about the concept, see [DNS firewalls](/docs/dns/concepts/dns-firewall).
type DnsFirewall struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the DNS firewall. Generated at creation time.
	Id string `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	// ID of the folder that the DNS firewall belongs to.
	FolderId string `protobuf:"bytes,2,opt,name=folder_id,json=folderId,proto3" json:"folder_id,omitempty"`
	// Creation timestamp.
	CreatedAt *timestamppb.Timestamp `protobuf:"bytes,3,opt,name=created_at,json=createdAt,proto3" json:"created_at,omitempty"`
	// Name of the DNS firewall.
	// The name is unique within the folder.
	Name string `protobuf:"bytes,4,opt,name=name,proto3" json:"name,omitempty"`
	// Description of the DNS firewall.
	Description string `protobuf:"bytes,5,opt,name=description,proto3" json:"description,omitempty"`
	// DNS firewall labels as `key:value` pairs.
	Labels map[string]string `protobuf:"bytes,6,rep,name=labels,proto3" json:"labels,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	// Prevents accidental firewall removal.
	DeletionProtection bool `protobuf:"varint,7,opt,name=deletion_protection,json=deletionProtection,proto3" json:"deletion_protection,omitempty"`
	// Whether the DNS firewall is enabled.
	Enabled bool `protobuf:"varint,8,opt,name=enabled,proto3" json:"enabled,omitempty"`
	// Resource settings.
	// Specifies resources that the DNS firewall applies to.
	ResourceConfig *DnsFirewall_ResourceConfig `protobuf:"bytes,9,opt,name=resource_config,json=resourceConfig,proto3" json:"resource_config,omitempty"`
	// List of FQDNs that the DNS firewall allows to pass.
	WhitelistFqdns []string `protobuf:"bytes,10,rep,name=whitelist_fqdns,json=whitelistFqdns,proto3" json:"whitelist_fqdns,omitempty"`
	// List of FQDNs that the DNS firewall blocks.
	BlacklistFqdns []string `protobuf:"bytes,11,rep,name=blacklist_fqdns,json=blacklistFqdns,proto3" json:"blacklist_fqdns,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *DnsFirewall) Reset() {
	*x = DnsFirewall{}
	mi := &file_yandex_cloud_dns_v1_dns_firewall_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DnsFirewall) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DnsFirewall) ProtoMessage() {}

func (x *DnsFirewall) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_dns_v1_dns_firewall_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DnsFirewall.ProtoReflect.Descriptor instead.
func (*DnsFirewall) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_dns_v1_dns_firewall_proto_rawDescGZIP(), []int{0}
}

func (x *DnsFirewall) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

func (x *DnsFirewall) GetFolderId() string {
	if x != nil {
		return x.FolderId
	}
	return ""
}

func (x *DnsFirewall) GetCreatedAt() *timestamppb.Timestamp {
	if x != nil {
		return x.CreatedAt
	}
	return nil
}

func (x *DnsFirewall) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *DnsFirewall) GetDescription() string {
	if x != nil {
		return x.Description
	}
	return ""
}

func (x *DnsFirewall) GetLabels() map[string]string {
	if x != nil {
		return x.Labels
	}
	return nil
}

func (x *DnsFirewall) GetDeletionProtection() bool {
	if x != nil {
		return x.DeletionProtection
	}
	return false
}

func (x *DnsFirewall) GetEnabled() bool {
	if x != nil {
		return x.Enabled
	}
	return false
}

func (x *DnsFirewall) GetResourceConfig() *DnsFirewall_ResourceConfig {
	if x != nil {
		return x.ResourceConfig
	}
	return nil
}

func (x *DnsFirewall) GetWhitelistFqdns() []string {
	if x != nil {
		return x.WhitelistFqdns
	}
	return nil
}

func (x *DnsFirewall) GetBlacklistFqdns() []string {
	if x != nil {
		return x.BlacklistFqdns
	}
	return nil
}

type DnsFirewall_ResourceConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Type of the resource.
	Type DnsFirewall_ResourceConfig_ResourceType `protobuf:"varint,1,opt,name=type,proto3,enum=yandex.cloud.dns.v1.DnsFirewall_ResourceConfig_ResourceType" json:"type,omitempty"`
	// Resource ids.
	ResourceIds []string `protobuf:"bytes,2,rep,name=resource_ids,json=resourceIds,proto3" json:"resource_ids,omitempty"`
	// Whether the locking policy is enabled.
	LockingPolicyEnabled bool `protobuf:"varint,3,opt,name=locking_policy_enabled,json=lockingPolicyEnabled,proto3" json:"locking_policy_enabled,omitempty"`
	unknownFields        protoimpl.UnknownFields
	sizeCache            protoimpl.SizeCache
}

func (x *DnsFirewall_ResourceConfig) Reset() {
	*x = DnsFirewall_ResourceConfig{}
	mi := &file_yandex_cloud_dns_v1_dns_firewall_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DnsFirewall_ResourceConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DnsFirewall_ResourceConfig) ProtoMessage() {}

func (x *DnsFirewall_ResourceConfig) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_dns_v1_dns_firewall_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DnsFirewall_ResourceConfig.ProtoReflect.Descriptor instead.
func (*DnsFirewall_ResourceConfig) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_dns_v1_dns_firewall_proto_rawDescGZIP(), []int{0, 0}
}

func (x *DnsFirewall_ResourceConfig) GetType() DnsFirewall_ResourceConfig_ResourceType {
	if x != nil {
		return x.Type
	}
	return DnsFirewall_ResourceConfig_RESOURCE_TYPE_UNSPECIFIED
}

func (x *DnsFirewall_ResourceConfig) GetResourceIds() []string {
	if x != nil {
		return x.ResourceIds
	}
	return nil
}

func (x *DnsFirewall_ResourceConfig) GetLockingPolicyEnabled() bool {
	if x != nil {
		return x.LockingPolicyEnabled
	}
	return false
}

var File_yandex_cloud_dns_v1_dns_firewall_proto protoreflect.FileDescriptor

const file_yandex_cloud_dns_v1_dns_firewall_proto_rawDesc = "" +
	"\n" +
	"&yandex/cloud/dns/v1/dns_firewall.proto\x12\x13yandex.cloud.dns.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1dyandex/cloud/validation.proto\"\xfb\x06\n" +
	"\vDnsFirewall\x12\x0e\n" +
	"\x02id\x18\x01 \x01(\tR\x02id\x12\x1b\n" +
	"\tfolder_id\x18\x02 \x01(\tR\bfolderId\x129\n" +
	"\n" +
	"created_at\x18\x03 \x01(\v2\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x12\n" +
	"\x04name\x18\x04 \x01(\tR\x04name\x12 \n" +
	"\vdescription\x18\x05 \x01(\tR\vdescription\x12D\n" +
	"\x06labels\x18\x06 \x03(\v2,.yandex.cloud.dns.v1.DnsFirewall.LabelsEntryR\x06labels\x12/\n" +
	"\x13deletion_protection\x18\a \x01(\bR\x12deletionProtection\x12\x18\n" +
	"\aenabled\x18\b \x01(\bR\aenabled\x12X\n" +
	"\x0fresource_config\x18\t \x01(\v2/.yandex.cloud.dns.v1.DnsFirewall.ResourceConfigR\x0eresourceConfig\x12?\n" +
	"\x0fwhitelist_fqdns\x18\n" +
	" \x03(\tB\x16\x82\xc81\x05<=255\x8a\xc81\x05<=255\x90\xc81\x01R\x0ewhitelistFqdns\x12?\n" +
	"\x0fblacklist_fqdns\x18\v \x03(\tB\x16\x82\xc81\x05<=255\x8a\xc81\x05<=255\x90\xc81\x01R\x0eblacklistFqdns\x1a\xa5\x02\n" +
	"\x0eResourceConfig\x12P\n" +
	"\x04type\x18\x01 \x01(\x0e2<.yandex.cloud.dns.v1.DnsFirewall.ResourceConfig.ResourceTypeR\x04type\x128\n" +
	"\fresource_ids\x18\x02 \x03(\tB\x15\x82\xc81\x050-100\x8a\xc81\x04<=64\x90\xc81\x01R\vresourceIds\x124\n" +
	"\x16locking_policy_enabled\x18\x03 \x01(\bR\x14lockingPolicyEnabled\"Q\n" +
	"\fResourceType\x12\x1d\n" +
	"\x19RESOURCE_TYPE_UNSPECIFIED\x10\x00\x12\v\n" +
	"\aNETWORK\x10\x01\x12\n" +
	"\n" +
	"\x06FOLDER\x10\x02\x12\t\n" +
	"\x05CLOUD\x10\x03\x1a9\n" +
	"\vLabelsEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\tR\x05value:\x028\x01BV\n" +
	"\x17yandex.cloud.api.dns.v1Z;github.com/yandex-cloud/go-genproto/yandex/cloud/dns/v1;dnsb\x06proto3"

var (
	file_yandex_cloud_dns_v1_dns_firewall_proto_rawDescOnce sync.Once
	file_yandex_cloud_dns_v1_dns_firewall_proto_rawDescData []byte
)

func file_yandex_cloud_dns_v1_dns_firewall_proto_rawDescGZIP() []byte {
	file_yandex_cloud_dns_v1_dns_firewall_proto_rawDescOnce.Do(func() {
		file_yandex_cloud_dns_v1_dns_firewall_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_yandex_cloud_dns_v1_dns_firewall_proto_rawDesc), len(file_yandex_cloud_dns_v1_dns_firewall_proto_rawDesc)))
	})
	return file_yandex_cloud_dns_v1_dns_firewall_proto_rawDescData
}

var file_yandex_cloud_dns_v1_dns_firewall_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_yandex_cloud_dns_v1_dns_firewall_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_yandex_cloud_dns_v1_dns_firewall_proto_goTypes = []any{
	(DnsFirewall_ResourceConfig_ResourceType)(0), // 0: yandex.cloud.dns.v1.DnsFirewall.ResourceConfig.ResourceType
	(*DnsFirewall)(nil),                          // 1: yandex.cloud.dns.v1.DnsFirewall
	(*DnsFirewall_ResourceConfig)(nil),           // 2: yandex.cloud.dns.v1.DnsFirewall.ResourceConfig
	nil,                                          // 3: yandex.cloud.dns.v1.DnsFirewall.LabelsEntry
	(*timestamppb.Timestamp)(nil),                // 4: google.protobuf.Timestamp
}
var file_yandex_cloud_dns_v1_dns_firewall_proto_depIdxs = []int32{
	4, // 0: yandex.cloud.dns.v1.DnsFirewall.created_at:type_name -> google.protobuf.Timestamp
	3, // 1: yandex.cloud.dns.v1.DnsFirewall.labels:type_name -> yandex.cloud.dns.v1.DnsFirewall.LabelsEntry
	2, // 2: yandex.cloud.dns.v1.DnsFirewall.resource_config:type_name -> yandex.cloud.dns.v1.DnsFirewall.ResourceConfig
	0, // 3: yandex.cloud.dns.v1.DnsFirewall.ResourceConfig.type:type_name -> yandex.cloud.dns.v1.DnsFirewall.ResourceConfig.ResourceType
	4, // [4:4] is the sub-list for method output_type
	4, // [4:4] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	4, // [4:4] is the sub-list for extension extendee
	0, // [0:4] is the sub-list for field type_name
}

func init() { file_yandex_cloud_dns_v1_dns_firewall_proto_init() }
func file_yandex_cloud_dns_v1_dns_firewall_proto_init() {
	if File_yandex_cloud_dns_v1_dns_firewall_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_yandex_cloud_dns_v1_dns_firewall_proto_rawDesc), len(file_yandex_cloud_dns_v1_dns_firewall_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_yandex_cloud_dns_v1_dns_firewall_proto_goTypes,
		DependencyIndexes: file_yandex_cloud_dns_v1_dns_firewall_proto_depIdxs,
		EnumInfos:         file_yandex_cloud_dns_v1_dns_firewall_proto_enumTypes,
		MessageInfos:      file_yandex_cloud_dns_v1_dns_firewall_proto_msgTypes,
	}.Build()
	File_yandex_cloud_dns_v1_dns_firewall_proto = out.File
	file_yandex_cloud_dns_v1_dns_firewall_proto_goTypes = nil
	file_yandex_cloud_dns_v1_dns_firewall_proto_depIdxs = nil
}
