// Code generated by protoc-gen-goext. DO NOT EDIT.

package dns

import (
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
)

func (m *DnsZone) SetId(v string) {
	m.Id = v
}

func (m *DnsZone) SetFolderId(v string) {
	m.FolderId = v
}

func (m *DnsZone) SetCreatedAt(v *timestamppb.Timestamp) {
	m.CreatedAt = v
}

func (m *DnsZone) SetName(v string) {
	m.Name = v
}

func (m *DnsZone) SetDescription(v string) {
	m.Description = v
}

func (m *DnsZone) SetLabels(v map[string]string) {
	m.Labels = v
}

func (m *DnsZone) SetZone(v string) {
	m.Zone = v
}

func (m *DnsZone) SetPrivateVisibility(v *PrivateVisibility) {
	m.PrivateVisibility = v
}

func (m *DnsZone) SetPublicVisibility(v *PublicVisibility) {
	m.PublicVisibility = v
}

func (m *DnsZone) SetDeletionProtection(v bool) {
	m.DeletionProtection = v
}

func (m *RecordSet) SetName(v string) {
	m.Name = v
}

func (m *RecordSet) SetType(v string) {
	m.Type = v
}

func (m *RecordSet) SetTtl(v int64) {
	m.Ttl = v
}

func (m *RecordSet) SetData(v []string) {
	m.Data = v
}

func (m *RecordSet) SetDescription(v string) {
	m.Description = v
}

func (m *PrivateVisibility) SetNetworkIds(v []string) {
	m.NetworkIds = v
}
