// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v3.21.12
// source: yandex/cloud/access/access.proto

package access

import (
	_ "github.com/yandex-cloud/go-genproto/yandex/cloud"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type AccessBindingAction int32

const (
	AccessBindingAction_ACCESS_BINDING_ACTION_UNSPECIFIED AccessBindingAction = 0
	// Addition of an access binding.
	AccessBindingAction_ADD AccessBindingAction = 1
	// Removal of an access binding.
	AccessBindingAction_REMOVE AccessBindingAction = 2
)

// Enum value maps for AccessBindingAction.
var (
	AccessBindingAction_name = map[int32]string{
		0: "ACCESS_BINDING_ACTION_UNSPECIFIED",
		1: "ADD",
		2: "REMOVE",
	}
	AccessBindingAction_value = map[string]int32{
		"ACCESS_BINDING_ACTION_UNSPECIFIED": 0,
		"ADD":                               1,
		"REMOVE":                            2,
	}
)

func (x AccessBindingAction) Enum() *AccessBindingAction {
	p := new(AccessBindingAction)
	*p = x
	return p
}

func (x AccessBindingAction) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (AccessBindingAction) Descriptor() protoreflect.EnumDescriptor {
	return file_yandex_cloud_access_access_proto_enumTypes[0].Descriptor()
}

func (AccessBindingAction) Type() protoreflect.EnumType {
	return &file_yandex_cloud_access_access_proto_enumTypes[0]
}

func (x AccessBindingAction) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use AccessBindingAction.Descriptor instead.
func (AccessBindingAction) EnumDescriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{0}
}

type Subject struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the subject.
	// It can contain one of the following values:oauth
	// * `allAuthenticatedUsers`: A special public group that represents anyone
	// who is authenticated. It can be used only if the [type] is `system`.
	// * `allUsers`: A special public group that represents anyone. No authentication is required.
	// For example, you don't need to specify the IAM token in an API query.
	// It can be used only if the [type] is `system`.
	// * `group:organization:<id>:users`: A special system group that represents all members of organization
	// with given <id>. It can be used only if the [type] is `system`.
	// * `group:federation:<id>:users`: A special system group that represents all users of federation
	// with given <id>. It can be used only if the [type] is `system`.
	// * `<cloud generated id>`: An identifier that represents a user account.
	// It can be used only if the [type] is `userAccount`, `federatedUser` or `serviceAccount`.
	Id string `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	// Type of the subject.
	// It can contain one of the following values:
	// * `userAccount`: An account on Yandex or Yandex Connect, added to Yandex Cloud.
	// * `serviceAccount`: A service account. This type represents the [yandex.cloud.iam.v1.ServiceAccount] resource.
	// * `federatedUser`: A federated account. This type represents a user from an identity federation, like Active Directory.
	// * `system`: System group. This type represents several accounts with a common system identifier.
	// For more information, see [Subject to which the role is assigned](/docs/iam/concepts/access-control/#subject).
	Type          string `protobuf:"bytes,2,opt,name=type,proto3" json:"type,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Subject) Reset() {
	*x = Subject{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Subject) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Subject) ProtoMessage() {}

func (x *Subject) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Subject.ProtoReflect.Descriptor instead.
func (*Subject) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{0}
}

func (x *Subject) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

func (x *Subject) GetType() string {
	if x != nil {
		return x.Type
	}
	return ""
}

type AccessBinding struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the [yandex.cloud.iam.v1.Role] that is assigned to the [subject].
	RoleId string `protobuf:"bytes,1,opt,name=role_id,json=roleId,proto3" json:"role_id,omitempty"`
	// Identity for which access binding is being created.
	// It can represent an account with a unique ID or several accounts with a system identifier.
	Subject       *Subject `protobuf:"bytes,2,opt,name=subject,proto3" json:"subject,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AccessBinding) Reset() {
	*x = AccessBinding{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AccessBinding) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AccessBinding) ProtoMessage() {}

func (x *AccessBinding) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AccessBinding.ProtoReflect.Descriptor instead.
func (*AccessBinding) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{1}
}

func (x *AccessBinding) GetRoleId() string {
	if x != nil {
		return x.RoleId
	}
	return ""
}

func (x *AccessBinding) GetSubject() *Subject {
	if x != nil {
		return x.Subject
	}
	return nil
}

type AccessPolicy struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the access policy template.
	Id string `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	// Name of the access policy template.
	Name string `protobuf:"bytes,2,opt,name=name,proto3" json:"name,omitempty"`
	// Description of the access policy template.
	Description   string `protobuf:"bytes,3,opt,name=description,proto3" json:"description,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AccessPolicy) Reset() {
	*x = AccessPolicy{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AccessPolicy) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AccessPolicy) ProtoMessage() {}

func (x *AccessPolicy) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AccessPolicy.ProtoReflect.Descriptor instead.
func (*AccessPolicy) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{2}
}

func (x *AccessPolicy) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

func (x *AccessPolicy) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *AccessPolicy) GetDescription() string {
	if x != nil {
		return x.Description
	}
	return ""
}

type AccessPolicyBinding struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the access policy template being applied.
	AccessPolicyTemplateId string `protobuf:"bytes,1,opt,name=access_policy_template_id,json=accessPolicyTemplateId,proto3" json:"access_policy_template_id,omitempty"`
	// A list of access policy binding parameter KEY=VALUE pairs.
	Parameters    map[string]string `protobuf:"bytes,2,rep,name=parameters,proto3" json:"parameters,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AccessPolicyBinding) Reset() {
	*x = AccessPolicyBinding{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AccessPolicyBinding) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AccessPolicyBinding) ProtoMessage() {}

func (x *AccessPolicyBinding) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AccessPolicyBinding.ProtoReflect.Descriptor instead.
func (*AccessPolicyBinding) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{3}
}

func (x *AccessPolicyBinding) GetAccessPolicyTemplateId() string {
	if x != nil {
		return x.AccessPolicyTemplateId
	}
	return ""
}

func (x *AccessPolicyBinding) GetParameters() map[string]string {
	if x != nil {
		return x.Parameters
	}
	return nil
}

type BindAccessPolicyRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the resource for which access policy bindings are being set.
	// To get the resource ID, use a corresponding List request.
	ResourceId string `protobuf:"bytes,1,opt,name=resource_id,json=resourceId,proto3" json:"resource_id,omitempty"`
	// Access policy binding. For more information, see [Access Bindings](/docs/iam/concepts/access-control/#access-policies).
	AccessPolicyBinding *AccessPolicyBinding `protobuf:"bytes,2,opt,name=access_policy_binding,json=accessPolicyBinding,proto3" json:"access_policy_binding,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *BindAccessPolicyRequest) Reset() {
	*x = BindAccessPolicyRequest{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *BindAccessPolicyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*BindAccessPolicyRequest) ProtoMessage() {}

func (x *BindAccessPolicyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use BindAccessPolicyRequest.ProtoReflect.Descriptor instead.
func (*BindAccessPolicyRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{4}
}

func (x *BindAccessPolicyRequest) GetResourceId() string {
	if x != nil {
		return x.ResourceId
	}
	return ""
}

func (x *BindAccessPolicyRequest) GetAccessPolicyBinding() *AccessPolicyBinding {
	if x != nil {
		return x.AccessPolicyBinding
	}
	return nil
}

type BindAccessPolicyMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the resource for which access policy bindings are being set.
	ResourceId string `protobuf:"bytes,1,opt,name=resource_id,json=resourceId,proto3" json:"resource_id,omitempty"`
	// Access policy binding. For more information, see [Access Bindings](/docs/iam/concepts/access-control/#access-policies).
	AccessPolicyBinding *AccessPolicyBinding `protobuf:"bytes,2,opt,name=access_policy_binding,json=accessPolicyBinding,proto3" json:"access_policy_binding,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *BindAccessPolicyMetadata) Reset() {
	*x = BindAccessPolicyMetadata{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *BindAccessPolicyMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*BindAccessPolicyMetadata) ProtoMessage() {}

func (x *BindAccessPolicyMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use BindAccessPolicyMetadata.ProtoReflect.Descriptor instead.
func (*BindAccessPolicyMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{5}
}

func (x *BindAccessPolicyMetadata) GetResourceId() string {
	if x != nil {
		return x.ResourceId
	}
	return ""
}

func (x *BindAccessPolicyMetadata) GetAccessPolicyBinding() *AccessPolicyBinding {
	if x != nil {
		return x.AccessPolicyBinding
	}
	return nil
}

type BindAccessPolicyResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *BindAccessPolicyResponse) Reset() {
	*x = BindAccessPolicyResponse{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *BindAccessPolicyResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*BindAccessPolicyResponse) ProtoMessage() {}

func (x *BindAccessPolicyResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use BindAccessPolicyResponse.ProtoReflect.Descriptor instead.
func (*BindAccessPolicyResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{6}
}

type ListAccessBindingsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the resource to list access bindings for.
	// To get the resource ID, use a corresponding List request.
	// For example, use the [yandex.cloud.resourcemanager.v1.CloudService.List] request to get the Cloud resource ID.
	ResourceId string `protobuf:"bytes,1,opt,name=resource_id,json=resourceId,proto3" json:"resource_id,omitempty"`
	// The maximum number of results per page that should be returned. If the number of available
	// results is larger than [page_size],
	// the service returns a [ListAccessBindingsResponse.next_page_token]
	// that can be used to get the next page of results in subsequent list requests.
	// Default value: 100.
	PageSize int64 `protobuf:"varint,2,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// Page token. Set [page_token]
	// to the [ListAccessBindingsResponse.next_page_token]
	// returned by a previous list request to get the next page of results.
	PageToken     string `protobuf:"bytes,3,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListAccessBindingsRequest) Reset() {
	*x = ListAccessBindingsRequest{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListAccessBindingsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListAccessBindingsRequest) ProtoMessage() {}

func (x *ListAccessBindingsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListAccessBindingsRequest.ProtoReflect.Descriptor instead.
func (*ListAccessBindingsRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{7}
}

func (x *ListAccessBindingsRequest) GetResourceId() string {
	if x != nil {
		return x.ResourceId
	}
	return ""
}

func (x *ListAccessBindingsRequest) GetPageSize() int64 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListAccessBindingsRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

type ListAccessBindingsResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// List of access bindings for the specified resource.
	AccessBindings []*AccessBinding `protobuf:"bytes,1,rep,name=access_bindings,json=accessBindings,proto3" json:"access_bindings,omitempty"`
	// This token allows you to get the next page of results for list requests. If the number of results
	// is larger than [ListAccessBindingsRequest.page_size], use
	// the [next_page_token] as the value
	// for the [ListAccessBindingsRequest.page_token] query parameter
	// in the next list request. Each subsequent list request will have its own
	// [next_page_token] to continue paging through the results.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListAccessBindingsResponse) Reset() {
	*x = ListAccessBindingsResponse{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListAccessBindingsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListAccessBindingsResponse) ProtoMessage() {}

func (x *ListAccessBindingsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListAccessBindingsResponse.ProtoReflect.Descriptor instead.
func (*ListAccessBindingsResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{8}
}

func (x *ListAccessBindingsResponse) GetAccessBindings() []*AccessBinding {
	if x != nil {
		return x.AccessBindings
	}
	return nil
}

func (x *ListAccessBindingsResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

type ListAccessPolicyBindingsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the resource to list access policy bindings for.
	// To get the resource ID, use a corresponding List request.
	// For example, use the [yandex.cloud.resourcemanager.v1.CloudService.List] request to get the Cloud resource ID.
	ResourceId string `protobuf:"bytes,1,opt,name=resource_id,json=resourceId,proto3" json:"resource_id,omitempty"`
	// The maximum number of results per page that should be returned.
	// If the number of available results is larger than [page_size], the service returns a
	// [ListAccessPolicyBindingsResponse.next_page_token]
	// that can be used to get the next page of results in subsequent list requests.
	// Default value: 100.
	PageSize int64 `protobuf:"varint,2,opt,name=page_size,json=pageSize,proto3" json:"page_size,omitempty"`
	// Page token. Set [page_token] to the
	// [ListAccessPolicyBindingsResponse.next_page_token]
	// returned by a previous list request to get the next page of results.
	PageToken     string `protobuf:"bytes,3,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListAccessPolicyBindingsRequest) Reset() {
	*x = ListAccessPolicyBindingsRequest{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[9]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListAccessPolicyBindingsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListAccessPolicyBindingsRequest) ProtoMessage() {}

func (x *ListAccessPolicyBindingsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[9]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListAccessPolicyBindingsRequest.ProtoReflect.Descriptor instead.
func (*ListAccessPolicyBindingsRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{9}
}

func (x *ListAccessPolicyBindingsRequest) GetResourceId() string {
	if x != nil {
		return x.ResourceId
	}
	return ""
}

func (x *ListAccessPolicyBindingsRequest) GetPageSize() int64 {
	if x != nil {
		return x.PageSize
	}
	return 0
}

func (x *ListAccessPolicyBindingsRequest) GetPageToken() string {
	if x != nil {
		return x.PageToken
	}
	return ""
}

type ListAccessPolicyBindingsResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// List of access policy bindings for the specified resource.
	AccessPolicyBindings []*AccessPolicyBinding `protobuf:"bytes,1,rep,name=access_policy_bindings,json=accessPolicyBindings,proto3" json:"access_policy_bindings,omitempty"`
	// This token allows you to get the next page of results for list requests.
	// If the number of results is larger than
	// [ListAccessPolicyBindingsRequest.page_size]
	// , use the [next_page_token] as the value for the
	// [ListAccessPolicyBindingsRequest.page_token]
	// query parameter in the next list request. Each subsequent list request will have its own
	// [next_page_token] to continue paging through the results.
	NextPageToken string `protobuf:"bytes,2,opt,name=next_page_token,json=nextPageToken,proto3" json:"next_page_token,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ListAccessPolicyBindingsResponse) Reset() {
	*x = ListAccessPolicyBindingsResponse{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[10]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ListAccessPolicyBindingsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ListAccessPolicyBindingsResponse) ProtoMessage() {}

func (x *ListAccessPolicyBindingsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[10]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ListAccessPolicyBindingsResponse.ProtoReflect.Descriptor instead.
func (*ListAccessPolicyBindingsResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{10}
}

func (x *ListAccessPolicyBindingsResponse) GetAccessPolicyBindings() []*AccessPolicyBinding {
	if x != nil {
		return x.AccessPolicyBindings
	}
	return nil
}

func (x *ListAccessPolicyBindingsResponse) GetNextPageToken() string {
	if x != nil {
		return x.NextPageToken
	}
	return ""
}

type UnbindAccessPolicyRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the resource for which access policy bindings are being removed.
	// To get the resource ID, use a corresponding List request.
	ResourceId string `protobuf:"bytes,1,opt,name=resource_id,json=resourceId,proto3" json:"resource_id,omitempty"`
	// ID of the access policy.
	AccessPolicyTemplateId string `protobuf:"bytes,2,opt,name=access_policy_template_id,json=accessPolicyTemplateId,proto3" json:"access_policy_template_id,omitempty"`
	unknownFields          protoimpl.UnknownFields
	sizeCache              protoimpl.SizeCache
}

func (x *UnbindAccessPolicyRequest) Reset() {
	*x = UnbindAccessPolicyRequest{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[11]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UnbindAccessPolicyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UnbindAccessPolicyRequest) ProtoMessage() {}

func (x *UnbindAccessPolicyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[11]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UnbindAccessPolicyRequest.ProtoReflect.Descriptor instead.
func (*UnbindAccessPolicyRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{11}
}

func (x *UnbindAccessPolicyRequest) GetResourceId() string {
	if x != nil {
		return x.ResourceId
	}
	return ""
}

func (x *UnbindAccessPolicyRequest) GetAccessPolicyTemplateId() string {
	if x != nil {
		return x.AccessPolicyTemplateId
	}
	return ""
}

type UnbindAccessPolicyMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the resource for which access policy bindings are being removed.
	ResourceId string `protobuf:"bytes,1,opt,name=resource_id,json=resourceId,proto3" json:"resource_id,omitempty"`
	// ID of the access policy.
	AccessPolicyTemplateId string `protobuf:"bytes,2,opt,name=access_policy_template_id,json=accessPolicyTemplateId,proto3" json:"access_policy_template_id,omitempty"`
	unknownFields          protoimpl.UnknownFields
	sizeCache              protoimpl.SizeCache
}

func (x *UnbindAccessPolicyMetadata) Reset() {
	*x = UnbindAccessPolicyMetadata{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[12]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UnbindAccessPolicyMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UnbindAccessPolicyMetadata) ProtoMessage() {}

func (x *UnbindAccessPolicyMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[12]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UnbindAccessPolicyMetadata.ProtoReflect.Descriptor instead.
func (*UnbindAccessPolicyMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{12}
}

func (x *UnbindAccessPolicyMetadata) GetResourceId() string {
	if x != nil {
		return x.ResourceId
	}
	return ""
}

func (x *UnbindAccessPolicyMetadata) GetAccessPolicyTemplateId() string {
	if x != nil {
		return x.AccessPolicyTemplateId
	}
	return ""
}

type UnbindAccessPolicyResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UnbindAccessPolicyResponse) Reset() {
	*x = UnbindAccessPolicyResponse{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[13]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UnbindAccessPolicyResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UnbindAccessPolicyResponse) ProtoMessage() {}

func (x *UnbindAccessPolicyResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[13]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UnbindAccessPolicyResponse.ProtoReflect.Descriptor instead.
func (*UnbindAccessPolicyResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{13}
}

type UpdateAccessPolicyBindingParametersRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the resource for which access policy bindings are being updated.
	ResourceId string `protobuf:"bytes,1,opt,name=resource_id,json=resourceId,proto3" json:"resource_id,omitempty"`
	// Identity for which access policy binding is being updated.
	AccessPolicyBinding *AccessPolicyBinding `protobuf:"bytes,2,opt,name=access_policy_binding,json=accessPolicyBinding,proto3" json:"access_policy_binding,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *UpdateAccessPolicyBindingParametersRequest) Reset() {
	*x = UpdateAccessPolicyBindingParametersRequest{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[14]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateAccessPolicyBindingParametersRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateAccessPolicyBindingParametersRequest) ProtoMessage() {}

func (x *UpdateAccessPolicyBindingParametersRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[14]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateAccessPolicyBindingParametersRequest.ProtoReflect.Descriptor instead.
func (*UpdateAccessPolicyBindingParametersRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{14}
}

func (x *UpdateAccessPolicyBindingParametersRequest) GetResourceId() string {
	if x != nil {
		return x.ResourceId
	}
	return ""
}

func (x *UpdateAccessPolicyBindingParametersRequest) GetAccessPolicyBinding() *AccessPolicyBinding {
	if x != nil {
		return x.AccessPolicyBinding
	}
	return nil
}

type UpdateAccessPolicyBindingParametersMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the resource for which access policy bindings are being updated.
	ResourceId string `protobuf:"bytes,1,opt,name=resource_id,json=resourceId,proto3" json:"resource_id,omitempty"`
	// Identity for which access policy binding is being updated.
	AccessPolicyBinding *AccessPolicyBinding `protobuf:"bytes,2,opt,name=access_policy_binding,json=accessPolicyBinding,proto3" json:"access_policy_binding,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *UpdateAccessPolicyBindingParametersMetadata) Reset() {
	*x = UpdateAccessPolicyBindingParametersMetadata{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[15]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateAccessPolicyBindingParametersMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateAccessPolicyBindingParametersMetadata) ProtoMessage() {}

func (x *UpdateAccessPolicyBindingParametersMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[15]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateAccessPolicyBindingParametersMetadata.ProtoReflect.Descriptor instead.
func (*UpdateAccessPolicyBindingParametersMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{15}
}

func (x *UpdateAccessPolicyBindingParametersMetadata) GetResourceId() string {
	if x != nil {
		return x.ResourceId
	}
	return ""
}

func (x *UpdateAccessPolicyBindingParametersMetadata) GetAccessPolicyBinding() *AccessPolicyBinding {
	if x != nil {
		return x.AccessPolicyBinding
	}
	return nil
}

type UpdateAccessPolicyBindingParametersResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateAccessPolicyBindingParametersResponse) Reset() {
	*x = UpdateAccessPolicyBindingParametersResponse{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[16]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateAccessPolicyBindingParametersResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateAccessPolicyBindingParametersResponse) ProtoMessage() {}

func (x *UpdateAccessPolicyBindingParametersResponse) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[16]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateAccessPolicyBindingParametersResponse.ProtoReflect.Descriptor instead.
func (*UpdateAccessPolicyBindingParametersResponse) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{16}
}

type SetAccessBindingsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the resource for which access bindings are being set.
	// To get the resource ID, use a corresponding List request.
	ResourceId string `protobuf:"bytes,1,opt,name=resource_id,json=resourceId,proto3" json:"resource_id,omitempty"`
	// Access bindings to be set. For more information, see [Access Bindings](/docs/iam/concepts/access-control/#access-bindings).
	AccessBindings []*AccessBinding `protobuf:"bytes,2,rep,name=access_bindings,json=accessBindings,proto3" json:"access_bindings,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *SetAccessBindingsRequest) Reset() {
	*x = SetAccessBindingsRequest{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[17]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SetAccessBindingsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SetAccessBindingsRequest) ProtoMessage() {}

func (x *SetAccessBindingsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[17]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SetAccessBindingsRequest.ProtoReflect.Descriptor instead.
func (*SetAccessBindingsRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{17}
}

func (x *SetAccessBindingsRequest) GetResourceId() string {
	if x != nil {
		return x.ResourceId
	}
	return ""
}

func (x *SetAccessBindingsRequest) GetAccessBindings() []*AccessBinding {
	if x != nil {
		return x.AccessBindings
	}
	return nil
}

type SetAccessBindingsMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the resource for which access bindings are being set.
	ResourceId    string `protobuf:"bytes,1,opt,name=resource_id,json=resourceId,proto3" json:"resource_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *SetAccessBindingsMetadata) Reset() {
	*x = SetAccessBindingsMetadata{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[18]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *SetAccessBindingsMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*SetAccessBindingsMetadata) ProtoMessage() {}

func (x *SetAccessBindingsMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[18]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use SetAccessBindingsMetadata.ProtoReflect.Descriptor instead.
func (*SetAccessBindingsMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{18}
}

func (x *SetAccessBindingsMetadata) GetResourceId() string {
	if x != nil {
		return x.ResourceId
	}
	return ""
}

type UpdateAccessBindingsRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the resource for which access bindings are being updated.
	ResourceId string `protobuf:"bytes,1,opt,name=resource_id,json=resourceId,proto3" json:"resource_id,omitempty"`
	// Updates to access bindings.
	AccessBindingDeltas []*AccessBindingDelta `protobuf:"bytes,2,rep,name=access_binding_deltas,json=accessBindingDeltas,proto3" json:"access_binding_deltas,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *UpdateAccessBindingsRequest) Reset() {
	*x = UpdateAccessBindingsRequest{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[19]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateAccessBindingsRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateAccessBindingsRequest) ProtoMessage() {}

func (x *UpdateAccessBindingsRequest) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[19]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateAccessBindingsRequest.ProtoReflect.Descriptor instead.
func (*UpdateAccessBindingsRequest) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{19}
}

func (x *UpdateAccessBindingsRequest) GetResourceId() string {
	if x != nil {
		return x.ResourceId
	}
	return ""
}

func (x *UpdateAccessBindingsRequest) GetAccessBindingDeltas() []*AccessBindingDelta {
	if x != nil {
		return x.AccessBindingDeltas
	}
	return nil
}

type UpdateAccessBindingsMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the resource for which access bindings are being updated.
	ResourceId    string `protobuf:"bytes,1,opt,name=resource_id,json=resourceId,proto3" json:"resource_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *UpdateAccessBindingsMetadata) Reset() {
	*x = UpdateAccessBindingsMetadata{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[20]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *UpdateAccessBindingsMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*UpdateAccessBindingsMetadata) ProtoMessage() {}

func (x *UpdateAccessBindingsMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[20]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use UpdateAccessBindingsMetadata.ProtoReflect.Descriptor instead.
func (*UpdateAccessBindingsMetadata) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{20}
}

func (x *UpdateAccessBindingsMetadata) GetResourceId() string {
	if x != nil {
		return x.ResourceId
	}
	return ""
}

type AccessBindingDelta struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The action that is being performed on an access binding.
	Action AccessBindingAction `protobuf:"varint,1,opt,name=action,proto3,enum=yandex.cloud.access.AccessBindingAction" json:"action,omitempty"`
	// Access binding. For more information, see [Access Bindings](/docs/iam/concepts/access-control/#access-bindings).
	AccessBinding *AccessBinding `protobuf:"bytes,2,opt,name=access_binding,json=accessBinding,proto3" json:"access_binding,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AccessBindingDelta) Reset() {
	*x = AccessBindingDelta{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[21]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AccessBindingDelta) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AccessBindingDelta) ProtoMessage() {}

func (x *AccessBindingDelta) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[21]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AccessBindingDelta.ProtoReflect.Descriptor instead.
func (*AccessBindingDelta) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{21}
}

func (x *AccessBindingDelta) GetAction() AccessBindingAction {
	if x != nil {
		return x.Action
	}
	return AccessBindingAction_ACCESS_BINDING_ACTION_UNSPECIFIED
}

func (x *AccessBindingDelta) GetAccessBinding() *AccessBinding {
	if x != nil {
		return x.AccessBinding
	}
	return nil
}

type AccessBindingsOperationResult struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Result access binding deltas.
	EffectiveDeltas []*AccessBindingDelta `protobuf:"bytes,1,rep,name=effective_deltas,json=effectiveDeltas,proto3" json:"effective_deltas,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *AccessBindingsOperationResult) Reset() {
	*x = AccessBindingsOperationResult{}
	mi := &file_yandex_cloud_access_access_proto_msgTypes[22]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AccessBindingsOperationResult) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AccessBindingsOperationResult) ProtoMessage() {}

func (x *AccessBindingsOperationResult) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_access_access_proto_msgTypes[22]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AccessBindingsOperationResult.ProtoReflect.Descriptor instead.
func (*AccessBindingsOperationResult) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_access_access_proto_rawDescGZIP(), []int{22}
}

func (x *AccessBindingsOperationResult) GetEffectiveDeltas() []*AccessBindingDelta {
	if x != nil {
		return x.EffectiveDeltas
	}
	return nil
}

var File_yandex_cloud_access_access_proto protoreflect.FileDescriptor

const file_yandex_cloud_access_access_proto_rawDesc = "" +
	"\n" +
	" yandex/cloud/access/access.proto\x12\x13yandex.cloud.access\x1a\x1dyandex/cloud/validation.proto\"K\n" +
	"\aSubject\x12\x1d\n" +
	"\x02id\x18\x01 \x01(\tB\r\xe8\xc71\x01\x8a\xc81\x05<=100R\x02id\x12!\n" +
	"\x04type\x18\x02 \x01(\tB\r\xe8\xc71\x01\x8a\xc81\x05<=100R\x04type\"t\n" +
	"\rAccessBinding\x12%\n" +
	"\arole_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\x06roleId\x12<\n" +
	"\asubject\x18\x02 \x01(\v2\x1c.yandex.cloud.access.SubjectB\x04\xe8\xc71\x01R\asubject\"T\n" +
	"\fAccessPolicy\x12\x0e\n" +
	"\x02id\x18\x01 \x01(\tR\x02id\x12\x12\n" +
	"\x04name\x18\x02 \x01(\tR\x04name\x12 \n" +
	"\vdescription\x18\x03 \x01(\tR\vdescription\"\xa8\x02\n" +
	"\x13AccessPolicyBinding\x12G\n" +
	"\x19access_policy_template_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\x16accessPolicyTemplateId\x12\x88\x01\n" +
	"\n" +
	"parameters\x18\x02 \x03(\v28.yandex.cloud.access.AccessPolicyBinding.ParametersEntryB.\x82\xc81\x04<=64\x8a\xc81\x06<=1024\xb2\xc81\x18\x12\x10[a-z][-_0-9a-z]*\x1a\x041-63R\n" +
	"parameters\x1a=\n" +
	"\x0fParametersEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\tR\x05value:\x028\x01\"\xac\x01\n" +
	"\x17BindAccessPolicyRequest\x12-\n" +
	"\vresource_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\n" +
	"resourceId\x12b\n" +
	"\x15access_policy_binding\x18\x02 \x01(\v2(.yandex.cloud.access.AccessPolicyBindingB\x04\xe8\xc71\x01R\x13accessPolicyBinding\"\x99\x01\n" +
	"\x18BindAccessPolicyMetadata\x12\x1f\n" +
	"\vresource_id\x18\x01 \x01(\tR\n" +
	"resourceId\x12\\\n" +
	"\x15access_policy_binding\x18\x02 \x01(\v2(.yandex.cloud.access.AccessPolicyBindingR\x13accessPolicyBinding\"\x1a\n" +
	"\x18BindAccessPolicyResponse\"\x9d\x01\n" +
	"\x19ListAccessBindingsRequest\x12-\n" +
	"\vresource_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\n" +
	"resourceId\x12'\n" +
	"\tpage_size\x18\x02 \x01(\x03B\n" +
	"\xfa\xc71\x06<=1000R\bpageSize\x12(\n" +
	"\n" +
	"page_token\x18\x03 \x01(\tB\t\x8a\xc81\x05<=100R\tpageToken\"\x91\x01\n" +
	"\x1aListAccessBindingsResponse\x12K\n" +
	"\x0faccess_bindings\x18\x01 \x03(\v2\".yandex.cloud.access.AccessBindingR\x0eaccessBindings\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"\xa3\x01\n" +
	"\x1fListAccessPolicyBindingsRequest\x12-\n" +
	"\vresource_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\n" +
	"resourceId\x12'\n" +
	"\tpage_size\x18\x02 \x01(\x03B\n" +
	"\xfa\xc71\x060-1000R\bpageSize\x12(\n" +
	"\n" +
	"page_token\x18\x03 \x01(\tB\t\x8a\xc81\x05<=100R\tpageToken\"\xaa\x01\n" +
	" ListAccessPolicyBindingsResponse\x12^\n" +
	"\x16access_policy_bindings\x18\x01 \x03(\v2(.yandex.cloud.access.AccessPolicyBindingR\x14accessPolicyBindings\x12&\n" +
	"\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken\"\x8b\x01\n" +
	"\x19UnbindAccessPolicyRequest\x12-\n" +
	"\vresource_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\n" +
	"resourceId\x12?\n" +
	"\x19access_policy_template_id\x18\x02 \x01(\tB\x04\xe8\xc71\x01R\x16accessPolicyTemplateId\"x\n" +
	"\x1aUnbindAccessPolicyMetadata\x12\x1f\n" +
	"\vresource_id\x18\x01 \x01(\tR\n" +
	"resourceId\x129\n" +
	"\x19access_policy_template_id\x18\x02 \x01(\tR\x16accessPolicyTemplateId\"\x1c\n" +
	"\x1aUnbindAccessPolicyResponse\"\xbf\x01\n" +
	"*UpdateAccessPolicyBindingParametersRequest\x12-\n" +
	"\vresource_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\n" +
	"resourceId\x12b\n" +
	"\x15access_policy_binding\x18\x02 \x01(\v2(.yandex.cloud.access.AccessPolicyBindingB\x04\xe8\xc71\x01R\x13accessPolicyBinding\"\xac\x01\n" +
	"+UpdateAccessPolicyBindingParametersMetadata\x12\x1f\n" +
	"\vresource_id\x18\x01 \x01(\tR\n" +
	"resourceId\x12\\\n" +
	"\x15access_policy_binding\x18\x02 \x01(\v2(.yandex.cloud.access.AccessPolicyBindingR\x13accessPolicyBinding\"-\n" +
	"+UpdateAccessPolicyBindingParametersResponse\"\xa2\x01\n" +
	"\x18SetAccessBindingsRequest\x12-\n" +
	"\vresource_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\n" +
	"resourceId\x12W\n" +
	"\x0faccess_bindings\x18\x02 \x03(\v2\".yandex.cloud.access.AccessBindingB\n" +
	"\x82\xc81\x06<=1000R\x0eaccessBindings\"<\n" +
	"\x19SetAccessBindingsMetadata\x12\x1f\n" +
	"\vresource_id\x18\x01 \x01(\tR\n" +
	"resourceId\"\xb5\x01\n" +
	"\x1bUpdateAccessBindingsRequest\x12-\n" +
	"\vresource_id\x18\x01 \x01(\tB\f\xe8\xc71\x01\x8a\xc81\x04<=50R\n" +
	"resourceId\x12g\n" +
	"\x15access_binding_deltas\x18\x02 \x03(\v2'.yandex.cloud.access.AccessBindingDeltaB\n" +
	"\x82\xc81\x061-1000R\x13accessBindingDeltas\"?\n" +
	"\x1cUpdateAccessBindingsMetadata\x12\x1f\n" +
	"\vresource_id\x18\x01 \x01(\tR\n" +
	"resourceId\"\xad\x01\n" +
	"\x12AccessBindingDelta\x12F\n" +
	"\x06action\x18\x01 \x01(\x0e2(.yandex.cloud.access.AccessBindingActionB\x04\xe8\xc71\x01R\x06action\x12O\n" +
	"\x0eaccess_binding\x18\x02 \x01(\v2\".yandex.cloud.access.AccessBindingB\x04\xe8\xc71\x01R\raccessBinding\"s\n" +
	"\x1dAccessBindingsOperationResult\x12R\n" +
	"\x10effective_deltas\x18\x01 \x03(\v2'.yandex.cloud.access.AccessBindingDeltaR\x0feffectiveDeltas*Q\n" +
	"\x13AccessBindingAction\x12%\n" +
	"!ACCESS_BINDING_ACTION_UNSPECIFIED\x10\x00\x12\a\n" +
	"\x03ADD\x10\x01\x12\n" +
	"\n" +
	"\x06REMOVE\x10\x02BY\n" +
	"\x17yandex.cloud.api.accessZ>github.com/yandex-cloud/go-genproto/yandex/cloud/access;accessb\x06proto3"

var (
	file_yandex_cloud_access_access_proto_rawDescOnce sync.Once
	file_yandex_cloud_access_access_proto_rawDescData []byte
)

func file_yandex_cloud_access_access_proto_rawDescGZIP() []byte {
	file_yandex_cloud_access_access_proto_rawDescOnce.Do(func() {
		file_yandex_cloud_access_access_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_yandex_cloud_access_access_proto_rawDesc), len(file_yandex_cloud_access_access_proto_rawDesc)))
	})
	return file_yandex_cloud_access_access_proto_rawDescData
}

var file_yandex_cloud_access_access_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_yandex_cloud_access_access_proto_msgTypes = make([]protoimpl.MessageInfo, 24)
var file_yandex_cloud_access_access_proto_goTypes = []any{
	(AccessBindingAction)(0),                            // 0: yandex.cloud.access.AccessBindingAction
	(*Subject)(nil),                                     // 1: yandex.cloud.access.Subject
	(*AccessBinding)(nil),                               // 2: yandex.cloud.access.AccessBinding
	(*AccessPolicy)(nil),                                // 3: yandex.cloud.access.AccessPolicy
	(*AccessPolicyBinding)(nil),                         // 4: yandex.cloud.access.AccessPolicyBinding
	(*BindAccessPolicyRequest)(nil),                     // 5: yandex.cloud.access.BindAccessPolicyRequest
	(*BindAccessPolicyMetadata)(nil),                    // 6: yandex.cloud.access.BindAccessPolicyMetadata
	(*BindAccessPolicyResponse)(nil),                    // 7: yandex.cloud.access.BindAccessPolicyResponse
	(*ListAccessBindingsRequest)(nil),                   // 8: yandex.cloud.access.ListAccessBindingsRequest
	(*ListAccessBindingsResponse)(nil),                  // 9: yandex.cloud.access.ListAccessBindingsResponse
	(*ListAccessPolicyBindingsRequest)(nil),             // 10: yandex.cloud.access.ListAccessPolicyBindingsRequest
	(*ListAccessPolicyBindingsResponse)(nil),            // 11: yandex.cloud.access.ListAccessPolicyBindingsResponse
	(*UnbindAccessPolicyRequest)(nil),                   // 12: yandex.cloud.access.UnbindAccessPolicyRequest
	(*UnbindAccessPolicyMetadata)(nil),                  // 13: yandex.cloud.access.UnbindAccessPolicyMetadata
	(*UnbindAccessPolicyResponse)(nil),                  // 14: yandex.cloud.access.UnbindAccessPolicyResponse
	(*UpdateAccessPolicyBindingParametersRequest)(nil),  // 15: yandex.cloud.access.UpdateAccessPolicyBindingParametersRequest
	(*UpdateAccessPolicyBindingParametersMetadata)(nil), // 16: yandex.cloud.access.UpdateAccessPolicyBindingParametersMetadata
	(*UpdateAccessPolicyBindingParametersResponse)(nil), // 17: yandex.cloud.access.UpdateAccessPolicyBindingParametersResponse
	(*SetAccessBindingsRequest)(nil),                    // 18: yandex.cloud.access.SetAccessBindingsRequest
	(*SetAccessBindingsMetadata)(nil),                   // 19: yandex.cloud.access.SetAccessBindingsMetadata
	(*UpdateAccessBindingsRequest)(nil),                 // 20: yandex.cloud.access.UpdateAccessBindingsRequest
	(*UpdateAccessBindingsMetadata)(nil),                // 21: yandex.cloud.access.UpdateAccessBindingsMetadata
	(*AccessBindingDelta)(nil),                          // 22: yandex.cloud.access.AccessBindingDelta
	(*AccessBindingsOperationResult)(nil),               // 23: yandex.cloud.access.AccessBindingsOperationResult
	nil,                                                 // 24: yandex.cloud.access.AccessPolicyBinding.ParametersEntry
}
var file_yandex_cloud_access_access_proto_depIdxs = []int32{
	1,  // 0: yandex.cloud.access.AccessBinding.subject:type_name -> yandex.cloud.access.Subject
	24, // 1: yandex.cloud.access.AccessPolicyBinding.parameters:type_name -> yandex.cloud.access.AccessPolicyBinding.ParametersEntry
	4,  // 2: yandex.cloud.access.BindAccessPolicyRequest.access_policy_binding:type_name -> yandex.cloud.access.AccessPolicyBinding
	4,  // 3: yandex.cloud.access.BindAccessPolicyMetadata.access_policy_binding:type_name -> yandex.cloud.access.AccessPolicyBinding
	2,  // 4: yandex.cloud.access.ListAccessBindingsResponse.access_bindings:type_name -> yandex.cloud.access.AccessBinding
	4,  // 5: yandex.cloud.access.ListAccessPolicyBindingsResponse.access_policy_bindings:type_name -> yandex.cloud.access.AccessPolicyBinding
	4,  // 6: yandex.cloud.access.UpdateAccessPolicyBindingParametersRequest.access_policy_binding:type_name -> yandex.cloud.access.AccessPolicyBinding
	4,  // 7: yandex.cloud.access.UpdateAccessPolicyBindingParametersMetadata.access_policy_binding:type_name -> yandex.cloud.access.AccessPolicyBinding
	2,  // 8: yandex.cloud.access.SetAccessBindingsRequest.access_bindings:type_name -> yandex.cloud.access.AccessBinding
	22, // 9: yandex.cloud.access.UpdateAccessBindingsRequest.access_binding_deltas:type_name -> yandex.cloud.access.AccessBindingDelta
	0,  // 10: yandex.cloud.access.AccessBindingDelta.action:type_name -> yandex.cloud.access.AccessBindingAction
	2,  // 11: yandex.cloud.access.AccessBindingDelta.access_binding:type_name -> yandex.cloud.access.AccessBinding
	22, // 12: yandex.cloud.access.AccessBindingsOperationResult.effective_deltas:type_name -> yandex.cloud.access.AccessBindingDelta
	13, // [13:13] is the sub-list for method output_type
	13, // [13:13] is the sub-list for method input_type
	13, // [13:13] is the sub-list for extension type_name
	13, // [13:13] is the sub-list for extension extendee
	0,  // [0:13] is the sub-list for field type_name
}

func init() { file_yandex_cloud_access_access_proto_init() }
func file_yandex_cloud_access_access_proto_init() {
	if File_yandex_cloud_access_access_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_yandex_cloud_access_access_proto_rawDesc), len(file_yandex_cloud_access_access_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   24,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_yandex_cloud_access_access_proto_goTypes,
		DependencyIndexes: file_yandex_cloud_access_access_proto_depIdxs,
		EnumInfos:         file_yandex_cloud_access_access_proto_enumTypes,
		MessageInfos:      file_yandex_cloud_access_access_proto_msgTypes,
	}.Build()
	File_yandex_cloud_access_access_proto = out.File
	file_yandex_cloud_access_access_proto_goTypes = nil
	file_yandex_cloud_access_access_proto_depIdxs = nil
}
