// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v3.21.12
// source: yandex/cloud/iam/v1/service_control.proto

package iam

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Service_Status int32

const (
	Service_STATUS_UNSPECIFIED Service_Status = 0
	// The service is enabled.
	Service_ENABLED Service_Status = 1
	// The service is paused.
	Service_PAUSED Service_Status = 2
	// The service is disabled.
	Service_DISABLED Service_Status = 3
	// The service is being enabled.
	Service_ENABLING Service_Status = 4
	// The service is being resumed.
	Service_RESUMING Service_Status = 5
	// The service is being paused.
	Service_PAUSING Service_Status = 6
	// The service is being disabled.
	Service_DISABLING Service_Status = 7
	// The service is in error state.
	Service_ERROR Service_Status = 8
	// The service could be auto enabled.
	Service_DEFAULT Service_Status = 9
)

// Enum value maps for Service_Status.
var (
	Service_Status_name = map[int32]string{
		0: "STATUS_UNSPECIFIED",
		1: "ENABLED",
		2: "PAUSED",
		3: "DISABLED",
		4: "ENABLING",
		5: "RESUMING",
		6: "PAUSING",
		7: "DISABLING",
		8: "ERROR",
		9: "DEFAULT",
	}
	Service_Status_value = map[string]int32{
		"STATUS_UNSPECIFIED": 0,
		"ENABLED":            1,
		"PAUSED":             2,
		"DISABLED":           3,
		"ENABLING":           4,
		"RESUMING":           5,
		"PAUSING":            6,
		"DISABLING":          7,
		"ERROR":              8,
		"DEFAULT":            9,
	}
)

func (x Service_Status) Enum() *Service_Status {
	p := new(Service_Status)
	*p = x
	return p
}

func (x Service_Status) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (Service_Status) Descriptor() protoreflect.EnumDescriptor {
	return file_yandex_cloud_iam_v1_service_control_proto_enumTypes[0].Descriptor()
}

func (Service_Status) Type() protoreflect.EnumType {
	return &file_yandex_cloud_iam_v1_service_control_proto_enumTypes[0]
}

func (x Service_Status) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use Service_Status.Descriptor instead.
func (Service_Status) EnumDescriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_service_control_proto_rawDescGZIP(), []int{0, 0}
}

// A Service.
type Service struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the service.
	ServiceId string `protobuf:"bytes,1,opt,name=service_id,json=serviceId,proto3" json:"service_id,omitempty"`
	// Resource that the service belongs to.
	Resource *Resource `protobuf:"bytes,2,opt,name=resource,proto3" json:"resource,omitempty"`
	// Time of the last status update of the service.
	UpdatedAt *timestamppb.Timestamp `protobuf:"bytes,3,opt,name=updated_at,json=updatedAt,proto3" json:"updated_at,omitempty"`
	// Current status of the service.
	Status        Service_Status `protobuf:"varint,4,opt,name=status,proto3,enum=yandex.cloud.iam.v1.Service_Status" json:"status,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Service) Reset() {
	*x = Service{}
	mi := &file_yandex_cloud_iam_v1_service_control_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Service) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Service) ProtoMessage() {}

func (x *Service) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_service_control_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Service.ProtoReflect.Descriptor instead.
func (*Service) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_service_control_proto_rawDescGZIP(), []int{0}
}

func (x *Service) GetServiceId() string {
	if x != nil {
		return x.ServiceId
	}
	return ""
}

func (x *Service) GetResource() *Resource {
	if x != nil {
		return x.Resource
	}
	return nil
}

func (x *Service) GetUpdatedAt() *timestamppb.Timestamp {
	if x != nil {
		return x.UpdatedAt
	}
	return nil
}

func (x *Service) GetStatus() Service_Status {
	if x != nil {
		return x.Status
	}
	return Service_STATUS_UNSPECIFIED
}

type ServiceAgent struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// ID of the agent service account.
	ServiceAccountId string `protobuf:"bytes,1,opt,name=service_account_id,json=serviceAccountId,proto3" json:"service_account_id,omitempty"`
	// ID of the service.
	ServiceId string `protobuf:"bytes,2,opt,name=service_id,json=serviceId,proto3" json:"service_id,omitempty"`
	// ID of the microservice.
	MicroserviceId string `protobuf:"bytes,3,opt,name=microservice_id,json=microserviceId,proto3" json:"microservice_id,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *ServiceAgent) Reset() {
	*x = ServiceAgent{}
	mi := &file_yandex_cloud_iam_v1_service_control_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceAgent) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceAgent) ProtoMessage() {}

func (x *ServiceAgent) ProtoReflect() protoreflect.Message {
	mi := &file_yandex_cloud_iam_v1_service_control_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceAgent.ProtoReflect.Descriptor instead.
func (*ServiceAgent) Descriptor() ([]byte, []int) {
	return file_yandex_cloud_iam_v1_service_control_proto_rawDescGZIP(), []int{1}
}

func (x *ServiceAgent) GetServiceAccountId() string {
	if x != nil {
		return x.ServiceAccountId
	}
	return ""
}

func (x *ServiceAgent) GetServiceId() string {
	if x != nil {
		return x.ServiceId
	}
	return ""
}

func (x *ServiceAgent) GetMicroserviceId() string {
	if x != nil {
		return x.MicroserviceId
	}
	return ""
}

var File_yandex_cloud_iam_v1_service_control_proto protoreflect.FileDescriptor

const file_yandex_cloud_iam_v1_service_control_proto_rawDesc = "" +
	"\n" +
	")yandex/cloud/iam/v1/service_control.proto\x12\x13yandex.cloud.iam.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\"yandex/cloud/iam/v1/resource.proto\"\xf5\x02\n" +
	"\aService\x12\x1d\n" +
	"\n" +
	"service_id\x18\x01 \x01(\tR\tserviceId\x129\n" +
	"\bresource\x18\x02 \x01(\v2\x1d.yandex.cloud.iam.v1.ResourceR\bresource\x129\n" +
	"\n" +
	"updated_at\x18\x03 \x01(\v2\x1a.google.protobuf.TimestampR\tupdatedAt\x12;\n" +
	"\x06status\x18\x04 \x01(\x0e2#.yandex.cloud.iam.v1.Service.StatusR\x06status\"\x97\x01\n" +
	"\x06Status\x12\x16\n" +
	"\x12STATUS_UNSPECIFIED\x10\x00\x12\v\n" +
	"\aENABLED\x10\x01\x12\n" +
	"\n" +
	"\x06PAUSED\x10\x02\x12\f\n" +
	"\bDISABLED\x10\x03\x12\f\n" +
	"\bENABLING\x10\x04\x12\f\n" +
	"\bRESUMING\x10\x05\x12\v\n" +
	"\aPAUSING\x10\x06\x12\r\n" +
	"\tDISABLING\x10\a\x12\t\n" +
	"\x05ERROR\x10\b\x12\v\n" +
	"\aDEFAULT\x10\t\"\x84\x01\n" +
	"\fServiceAgent\x12,\n" +
	"\x12service_account_id\x18\x01 \x01(\tR\x10serviceAccountId\x12\x1d\n" +
	"\n" +
	"service_id\x18\x02 \x01(\tR\tserviceId\x12'\n" +
	"\x0fmicroservice_id\x18\x03 \x01(\tR\x0emicroserviceIdBV\n" +
	"\x17yandex.cloud.api.iam.v1Z;github.com/yandex-cloud/go-genproto/yandex/cloud/iam/v1;iamb\x06proto3"

var (
	file_yandex_cloud_iam_v1_service_control_proto_rawDescOnce sync.Once
	file_yandex_cloud_iam_v1_service_control_proto_rawDescData []byte
)

func file_yandex_cloud_iam_v1_service_control_proto_rawDescGZIP() []byte {
	file_yandex_cloud_iam_v1_service_control_proto_rawDescOnce.Do(func() {
		file_yandex_cloud_iam_v1_service_control_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_service_control_proto_rawDesc), len(file_yandex_cloud_iam_v1_service_control_proto_rawDesc)))
	})
	return file_yandex_cloud_iam_v1_service_control_proto_rawDescData
}

var file_yandex_cloud_iam_v1_service_control_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_yandex_cloud_iam_v1_service_control_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_yandex_cloud_iam_v1_service_control_proto_goTypes = []any{
	(Service_Status)(0),           // 0: yandex.cloud.iam.v1.Service.Status
	(*Service)(nil),               // 1: yandex.cloud.iam.v1.Service
	(*ServiceAgent)(nil),          // 2: yandex.cloud.iam.v1.ServiceAgent
	(*Resource)(nil),              // 3: yandex.cloud.iam.v1.Resource
	(*timestamppb.Timestamp)(nil), // 4: google.protobuf.Timestamp
}
var file_yandex_cloud_iam_v1_service_control_proto_depIdxs = []int32{
	3, // 0: yandex.cloud.iam.v1.Service.resource:type_name -> yandex.cloud.iam.v1.Resource
	4, // 1: yandex.cloud.iam.v1.Service.updated_at:type_name -> google.protobuf.Timestamp
	0, // 2: yandex.cloud.iam.v1.Service.status:type_name -> yandex.cloud.iam.v1.Service.Status
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() { file_yandex_cloud_iam_v1_service_control_proto_init() }
func file_yandex_cloud_iam_v1_service_control_proto_init() {
	if File_yandex_cloud_iam_v1_service_control_proto != nil {
		return
	}
	file_yandex_cloud_iam_v1_resource_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_yandex_cloud_iam_v1_service_control_proto_rawDesc), len(file_yandex_cloud_iam_v1_service_control_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_yandex_cloud_iam_v1_service_control_proto_goTypes,
		DependencyIndexes: file_yandex_cloud_iam_v1_service_control_proto_depIdxs,
		EnumInfos:         file_yandex_cloud_iam_v1_service_control_proto_enumTypes,
		MessageInfos:      file_yandex_cloud_iam_v1_service_control_proto_msgTypes,
	}.Build()
	File_yandex_cloud_iam_v1_service_control_proto = out.File
	file_yandex_cloud_iam_v1_service_control_proto_goTypes = nil
	file_yandex_cloud_iam_v1_service_control_proto_depIdxs = nil
}
