// Copyright 2018 JDCLOUD.COM
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// NOTE: This class is auto generated by the jdcloud code generator program.

package models

type DomainInfo struct {

	/* 域名的唯一ID (Optional) */
	Id int `json:"id"`

	/* 域名字符串 (Optional) */
	DomainName string `json:"domainName"`

	/* 创建时间，格式Unix timestamp，时间单位：毫秒 (Optional) */
	CreateTime int64 `json:"createTime"`

	/* 过期时间，格式Unix timestamp，时间单位：毫秒 (Optional) */
	ExpirationDate int64 `json:"expirationDate"`

	/* 套餐类型，免费:0 企业版:1 企业高级版:2 (Optional) */
	PackId int `json:"packId"`

	/* 套餐的名字 (Optional) */
	PackName string `json:"packName"`

	/* 解析的状态, 暂无解析:1，正常解析:2， 部分解析:3， 暂停解析:4 NS未修改:5，域名探测错误:7，域名未注册:8，注册局暂停解析:9 (Optional) */
	ResolvingStatus string `json:"resolvingStatus"`

	/* 创建者 (Optional) */
	Creator string `json:"creator"`

	/* 是否是京东云资源 (Optional) */
	JcloudNs bool `json:"jcloudNs"`

	/* 域名的锁定状态，0:未锁定， 1:已锁定 (Optional) */
	LockStatus int `json:"lockStatus"`

	/* 主域名当前的Nameserver列表 (Optional) */
	ProbeNsList []string `json:"probeNsList"`

	/* 主域名应该设置的Nameserver列表 (Optional) */
	DefNsList []string `json:"defNsList"`
}
