/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

import (
	metav1 "k8s.io/client-go/applyconfigurations/meta/v1"
	apisv1 "sigs.k8s.io/gateway-api/apis/v1"
)

// ListenerNamespacesApplyConfiguration represents a declarative configuration of the ListenerNamespaces type for use
// with apply.
//
// ListenerNamespaces indicate which namespaces ListenerSets should be selected from.
type ListenerNamespacesApplyConfiguration struct {
	// From indicates where ListenerSets can attach to this Gateway. Possible
	// values are:
	//
	// * Same: Only ListenerSets in the same namespace may be attached to this Gateway.
	// * Selector: ListenerSets in namespaces selected by the selector may be attached to this Gateway.
	// * All: ListenerSets in all namespaces may be attached to this Gateway.
	// * None: Only listeners defined in the Gateway's spec are allowed
	//
	// The default value None
	From *apisv1.FromNamespaces `json:"from,omitempty"`
	// Selector must be specified when From is set to "Selector". In that case,
	// only ListenerSets in Namespaces matching this Selector will be selected by this
	// Gateway. This field is ignored for other values of "From".
	Selector *metav1.LabelSelectorApplyConfiguration `json:"selector,omitempty"`
}

// ListenerNamespacesApplyConfiguration constructs a declarative configuration of the ListenerNamespaces type for use with
// apply.
func ListenerNamespaces() *ListenerNamespacesApplyConfiguration {
	return &ListenerNamespacesApplyConfiguration{}
}

// WithFrom sets the From field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the From field is set to the value of the last call.
func (b *ListenerNamespacesApplyConfiguration) WithFrom(value apisv1.FromNamespaces) *ListenerNamespacesApplyConfiguration {
	b.From = &value
	return b
}

// WithSelector sets the Selector field in the declarative configuration to the given value
// and returns the receiver, so that objects can be built by chaining "With" function invocations.
// If called multiple times, the Selector field is set to the value of the last call.
func (b *ListenerNamespacesApplyConfiguration) WithSelector(value *metav1.LabelSelectorApplyConfiguration) *ListenerNamespacesApplyConfiguration {
	b.Selector = value
	return b
}
