/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by applyconfiguration-gen. DO NOT EDIT.

package v1

// GRPCAuthConfigApplyConfiguration represents a declarative configuration of the GRPCAuthConfig type for use
// with apply.
//
// GRPCAuthConfig contains configuration for communication with Auth server
// backends that speak Envoy's ext_authz gRPC protocol.
//
// Requests and responses are defined in the protobufs explained at:
// https://www.envoyproxy.io/docs/envoy/latest/api-v3/service/auth/v3/external_auth.proto
type GRPCAuthConfigApplyConfiguration struct {
	// AllowedRequestHeaders specifies what headers from the client request
	// will be sent to the authorization server.
	//
	// If this list is empty, then all headers must be sent.
	//
	// If the list has entries, only those entries must be sent.
	AllowedRequestHeaders []string `json:"allowedHeaders,omitempty"`
}

// GRPCAuthConfigApplyConfiguration constructs a declarative configuration of the GRPCAuthConfig type for use with
// apply.
func GRPCAuthConfig() *GRPCAuthConfigApplyConfiguration {
	return &GRPCAuthConfigApplyConfiguration{}
}

// WithAllowedRequestHeaders adds the given value to the AllowedRequestHeaders field in the declarative configuration
// and returns the receiver, so that objects can be build by chaining "With" function invocations.
// If called multiple times, values provided by each call will be appended to the AllowedRequestHeaders field.
func (b *GRPCAuthConfigApplyConfiguration) WithAllowedRequestHeaders(values ...string) *GRPCAuthConfigApplyConfiguration {
	for i := range values {
		b.AllowedRequestHeaders = append(b.AllowedRequestHeaders, values[i])
	}
	return b
}
