// Copyright 2018 JDCLOUD.COM
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// NOTE: This class is auto generated by the jdcloud code generator program.

package models

type HostRRlb struct {

	/* 解析记录的值 (Optional) */
	HostValue string `json:"hostValue"`

	/* 解析记录的ID (Optional) */
	Id int `json:"id"`

	/* 解析记录的权重 (Optional) */
	Weight int `json:"weight"`

	/* 此条记录在总均衡中的比率的100倍 (Optional) */
	Rate float64 `json:"rate"`
}
